# OpenADMIXTURE.jl

This software package is an open-source Julia reimplementation of the [ADMIXTURE](https://dalexander.github.io/admixture/) package. With more efficient multi-threading scheme, it is 8 times faster than the original software. It also supports computation on an Nvidia CUDA GPU. By directly using the data format of the PLINK BED file, the memory usage is 16x-32x smaller than using `Float32` or `Float64` type.

It estimates ancestry with maximum-likelihood method for a large SNP genotype datasets, where individuals are assumed to be unrelated. The input is binary PLINK 1 BED-formatted file (`.bed`). Also, you will need an idea of $K$, the number of ancestral populations. If the number of SNPs is too large, you may choose to run on a subset of SNPs selected by their information content, using the [sparse $K$-means via feature ranking](https://github.com/kose-y/SparseKmeansFeatureRanking.jl) (SKFR) method.

## Installation

This package requires Julia v1.6 or later, which can be obtained from
<https://julialang.org/downloads/> or by building Julia from the sources in the
<https://github.com/JuliaLang/julia> repository.

The package can be installed by running the following code:
```julia
using Pkg
pkg"add https://github.com/kose-y/SparseKmeansFeatureRanking.jl"
pkg"add https://github.com/OpenMendel/OpenADMIXTURE.jl"
```
For running the examples below, the following are also necessary. 
```julia
pkg"add SnpArrays CSV DelimitedFiles"
```

For GPU support, an Nvidia GPU is required. Also, the following package has to be installed:
```julia
pkg"add CUDA"
```

## Basic Usage

We first import necessary packages:


```julia
using LinearAlgebra, Random, SnpArrays
using OpenADMIXTURE
using CSV, DelimitedFiles

```

We will use the PLINK file included in the `SnpArrays` package, whose path is obtained by:


```julia
filename = SnpArrays.datadir("EUR_subset.bed");
```

This file contains information on 54051 single nucleotide polymorphisms (SNP)s of 379 samples. The main driver function for admixture proportion estimation is called `run_admixture()`. 


```julia
d, clusters, aims = OpenADMIXTURE.run_admixture(filename, 4; T=Float64, use_gpu=false, rng=MersenneTwister(7856))
```

      0.087914 seconds (4 allocations: 64 bytes)
      0.092882 seconds (4 allocations: 64 bytes)
      0.088550 seconds (4 allocations: 64 bytes)
      0.092690 seconds (4 allocations: 64 bytes)
      0.088199 seconds (4 allocations: 64 bytes)
      0.092607 seconds (4 allocations: 64 bytes)
      0.087901 seconds (4 allocations: 64 bytes)
      0.092424 seconds (4 allocations: 64 bytes)
      0.088549 seconds (4 allocations: 64 bytes)
      0.093026 seconds (4 allocations: 64 bytes)
     22.884278 seconds (34.59 M allocations: 1.742 GiB, 3.08% gc time, 95.16% compilation time)
    initial ll: -1.5844784297993343e7
    -1.5844784297993343e7
      0.281655 seconds (4 allocations: 64 bytes)
      2.816971 seconds (3.95 M allocations: 210.701 MiB, 9.53% gc time, 99.98% compilation time)
      0.370153 seconds (4 allocations: 64 bytes)
      0.097757 seconds (25.01 k allocations: 1.334 MiB, 64.46% compilation time)
    -1.5844784297993343e7
      0.277199 seconds (4 allocations: 64 bytes)
      0.000504 seconds (2.66 k allocations: 125.109 KiB)
      0.369947 seconds (4 allocations: 64 bytes)
      0.033502 seconds (8 allocations: 768 bytes)
    -1.5844784297993343e7
    -1.5658797780233778e7
    -1.5661966737375751e7
    Iteration 1: ll = -1.5661966737375751e7, reldiff = 0.011538027730724302
      4.664041 seconds (4.00 M allocations: 217.843 MiB, 5.76% gc time, 62.15% compilation time)
    
    
    -1.5661966737375751e7
      0.277490 seconds (4 allocations: 64 bytes)
      0.000491 seconds (2.66 k allocations: 125.109 KiB)
      0.369987 seconds (4 allocations: 64 bytes)
      0.031646 seconds (8 allocations: 768 bytes)
    -1.5661966737375751e7
      0.280284 seconds (4 allocations: 64 bytes)
      0.000493 seconds (2.66 k allocations: 125.109 KiB)
      0.368922 seconds (4 allocations: 64 bytes)
      0.030659 seconds (8 allocations: 768 bytes)
    -1.5661966737375751e7
    -1.5609040036582384e7
    -1.5605199791986194e7
    Iteration 2: ll = -1.5605199791986194e7, reldiff = 0.0036245093825980914
      1.758175 seconds (8.49 k allocations: 7.039 MiB)
    
    
    -1.5605199791986194e7
      0.277546 seconds (4 allocations: 64 bytes)
      0.000498 seconds (2.66 k allocations: 125.109 KiB)
      0.369400 seconds (4 allocations: 64 bytes)
      0.029828 seconds (8 allocations: 768 bytes)
    -1.5605199791986194e7
      0.277210 seconds (4 allocations: 64 bytes)
      0.000967 seconds (2.66 k allocations: 125.109 KiB)
      0.371205 seconds (4 allocations: 64 bytes)
      0.028913 seconds (8 allocations: 768 bytes)
    -1.5605199791986194e7
    -1.558989311248903e7
    -1.5586228372093504e7
    Iteration 3: ll = -1.5586228372093504e7, reldiff = 0.0012157114388520478
      1.755166 seconds (8.49 k allocations: 8.697 MiB)
    
    
    -1.5586228372093504e7
      0.277433 seconds (4 allocations: 64 bytes)
      0.000543 seconds (2.66 k allocations: 125.109 KiB)
      0.369406 seconds (4 allocations: 64 bytes)
      0.028202 seconds (8 allocations: 768 bytes)
    -1.5586228372093504e7
      0.277489 seconds (4 allocations: 64 bytes)
      0.000514 seconds (2.66 k allocations: 125.109 KiB)
      0.369718 seconds (4 allocations: 64 bytes)
      0.027662 seconds (8 allocations: 768 bytes)
    -1.5586228372093504e7
    -1.557593892598145e7
    -1.557902813062439e7
    Iteration 4: ll = -1.557902813062439e7, reldiff = 0.00046196175862575915
      1.750993 seconds (8.49 k allocations: 8.697 MiB)
    
    
    -1.557902813062439e7
      0.278042 seconds (4 allocations: 64 bytes)
      0.000524 seconds (2.66 k allocations: 125.109 KiB)
      0.371190 seconds (4 allocations: 64 bytes)
      0.032144 seconds (8 allocations: 768 bytes)
    -1.557902813062439e7
      0.402467 seconds (4 allocations: 64 bytes)
      0.000528 seconds (2.66 k allocations: 125.109 KiB)
      0.369614 seconds (4 allocations: 64 bytes)
      0.027682 seconds (8 allocations: 768 bytes)
    -1.557902813062439e7
    -1.5566929427850641e7
    -1.5567066087137524e7
    Iteration 5: ll = -1.5567066087137524e7, reldiff = 0.0007678298919912029
      1.966889 seconds (8.49 k allocations: 8.697 MiB)
    
    
    -1.5567066087137524e7
      0.346047 seconds (4 allocations: 64 bytes)
      0.000535 seconds (2.66 k allocations: 125.109 KiB)
      0.369873 seconds (4 allocations: 64 bytes)
      0.027421 seconds (8 allocations: 768 bytes)
    -1.5567066087137524e7
      0.276677 seconds (4 allocations: 64 bytes)
      0.000541 seconds (2.66 k allocations: 125.109 KiB)
      0.368943 seconds (4 allocations: 64 bytes)
      0.027093 seconds (8 allocations: 768 bytes)
    -1.5567066087137524e7
    -1.556264024120358e7
    -1.5561476004841125e7
    Iteration 6: ll = -1.5561476004841125e7, reldiff = 0.000359096715149059
      1.821452 seconds (8.49 k allocations: 8.701 MiB)
    
    
    -1.5561476004841125e7
      0.279316 seconds (4 allocations: 64 bytes)
      0.000554 seconds (2.66 k allocations: 125.109 KiB)
      0.369430 seconds (4 allocations: 64 bytes)
      0.027207 seconds (8 allocations: 768 bytes)
    -1.5561476004841125e7
      0.277530 seconds (4 allocations: 64 bytes)
      0.000546 seconds (2.66 k allocations: 125.109 KiB)
      0.369106 seconds (4 allocations: 64 bytes)
      0.026839 seconds (8 allocations: 768 bytes)
    -1.5561476004841125e7
    -1.5558483191215409e7
    -1.5558616674667932e7
    Iteration 7: ll = -1.5558616674667932e7, reldiff = 0.00018374414948190118
      1.751149 seconds (8.49 k allocations: 8.697 MiB)
    
    
    -1.5558616674667932e7
      0.277001 seconds (4 allocations: 64 bytes)
      0.000557 seconds (2.66 k allocations: 125.109 KiB)
      0.369540 seconds (4 allocations: 64 bytes)
      0.026845 seconds (8 allocations: 768 bytes)
    -1.5558616674667932e7
      0.278763 seconds (4 allocations: 64 bytes)
      0.000558 seconds (2.66 k allocations: 125.109 KiB)
      0.369124 seconds (4 allocations: 64 bytes)
      0.026629 seconds (8 allocations: 768 bytes)
    -1.5558616674667932e7
    -1.555656394773972e7
    -1.5556607054798864e7
    Iteration 8: ll = -1.5556607054798864e7, reldiff = 0.00012916443094457538
      1.901857 seconds (8.49 k allocations: 8.697 MiB, 8.00% gc time)
    
    
    -1.5556607054798864e7
      0.280088 seconds (4 allocations: 64 bytes)
      0.000560 seconds (2.66 k allocations: 125.109 KiB)
      0.369409 seconds (4 allocations: 64 bytes)
      0.028105 seconds (8 allocations: 768 bytes)
    -1.5556607054798864e7
      0.277509 seconds (4 allocations: 64 bytes)
      0.000579 seconds (2.66 k allocations: 125.109 KiB)
      0.370149 seconds (4 allocations: 64 bytes)
      0.026658 seconds (8 allocations: 768 bytes)
    -1.5556607054798864e7
    -1.5555272556777712e7
    -1.5555028877712639e7
    Iteration 9: ll = -1.5555028877712639e7, reldiff = 0.00010144738378141057
      1.752803 seconds (8.49 k allocations: 8.697 MiB)
    
    
    -1.5555028877712639e7
      0.275904 seconds (4 allocations: 64 bytes)
      0.000563 seconds (2.66 k allocations: 125.109 KiB)
      0.368746 seconds (4 allocations: 64 bytes)
      0.026281 seconds (8 allocations: 768 bytes)
    -1.5555028877712639e7
      0.276656 seconds (4 allocations: 64 bytes)
      0.000562 seconds (2.66 k allocations: 125.109 KiB)
      0.369720 seconds (4 allocations: 64 bytes)
      0.026309 seconds (8 allocations: 768 bytes)
    -1.5555028877712639e7
    -1.5554389948288705e7
    -1.5572307162296195e7
    Iteration 10: ll = -1.5554389948288705e7, reldiff = 4.107542512179528e-5
      1.783213 seconds (8.49 k allocations: 8.697 MiB)
    
    
    -1.5554389948288705e7
      0.314982 seconds (4 allocations: 64 bytes)
      0.000567 seconds (2.66 k allocations: 125.109 KiB)
      0.367198 seconds (4 allocations: 64 bytes)
      0.026479 seconds (8 allocations: 768 bytes)
    -1.5554389948288705e7
      0.277563 seconds (4 allocations: 64 bytes)
      0.000566 seconds (2.66 k allocations: 125.109 KiB)
      0.368985 seconds (4 allocations: 64 bytes)
      0.026321 seconds (8 allocations: 768 bytes)
    -1.5554389948288705e7
    -1.5553867013950767e7
    -1.5562818039977925e7
    Iteration 11: ll = -1.5553867013950767e7, reldiff = 3.361972662873465e-5
      1.782861 seconds (8.49 k allocations: 8.697 MiB)
    
    
    -1.5553867013950767e7
      0.277292 seconds (4 allocations: 64 bytes)
      0.000597 seconds (2.66 k allocations: 125.109 KiB)
      0.369716 seconds (4 allocations: 64 bytes)
      0.026243 seconds (8 allocations: 768 bytes)
    -1.5553867013950767e7
      0.277867 seconds (4 allocations: 64 bytes)
      0.000576 seconds (2.66 k allocations: 125.109 KiB)
      0.369590 seconds (4 allocations: 64 bytes)
      0.026170 seconds (8 allocations: 768 bytes)
    -1.5553867013950767e7
    -1.5553428107705034e7
    -1.5559131012878079e7
    Iteration 12: ll = -1.5553428107705034e7, reldiff = 2.8218464600411636e-5
      1.748249 seconds (8.49 k allocations: 8.697 MiB)
    
    
    -1.5553428107705034e7
      0.277564 seconds (4 allocations: 64 bytes)
      0.000570 seconds (2.66 k allocations: 125.109 KiB)
      0.369791 seconds (4 allocations: 64 bytes)
      0.026247 seconds (8 allocations: 768 bytes)
    -1.5553428107705034e7
      0.276816 seconds (4 allocations: 64 bytes)
      0.000571 seconds (2.66 k allocations: 125.109 KiB)
      0.370342 seconds (4 allocations: 64 bytes)
      0.026179 seconds (8 allocations: 768 bytes)
    -1.5553428107705034e7
    -1.5553067050573548e7
    -1.5554525394768834e7
    Iteration 13: ll = -1.5553067050573548e7, reldiff = 2.3213990445434272e-5
      1.748259 seconds (8.49 k allocations: 8.697 MiB)
    
    
    -1.5553067050573548e7
      0.277551 seconds (4 allocations: 64 bytes)
      0.001085 seconds (2.66 k allocations: 125.109 KiB)
      0.371064 seconds (4 allocations: 64 bytes)
      0.026033 seconds (8 allocations: 768 bytes)
    -1.5553067050573548e7
      0.277290 seconds (4 allocations: 64 bytes)
      0.000573 seconds (2.66 k allocations: 125.109 KiB)
      0.376844 seconds (4 allocations: 64 bytes)
      0.026263 seconds (8 allocations: 768 bytes)
    -1.5553067050573548e7
    -1.555278497410975e7
    -1.5553105606121972e7
    Iteration 14: ll = -1.555278497410975e7, reldiff = 1.8136388332978923e-5
      1.756941 seconds (8.49 k allocations: 8.713 MiB)
    
    
    -1.555278497410975e7
      0.284532 seconds (4 allocations: 64 bytes)
      0.000573 seconds (2.66 k allocations: 125.109 KiB)
      0.367819 seconds (4 allocations: 64 bytes)
      0.026059 seconds (8 allocations: 768 bytes)
    -1.555278497410975e7
      0.276429 seconds (4 allocations: 64 bytes)
      0.000572 seconds (2.66 k allocations: 125.109 KiB)
      0.369100 seconds (4 allocations: 64 bytes)
      0.026108 seconds (8 allocations: 768 bytes)
    -1.555278497410975e7
    -1.5552595414835572e7
    -1.5552639211087858e7
    Iteration 15: ll = -1.5552639211087858e7, reldiff = 9.372149241089872e-6
      1.750865 seconds (8.49 k allocations: 8.697 MiB)
    
    
    -1.5552639211087858e7
      0.277442 seconds (4 allocations: 64 bytes)
      0.000793 seconds (2.66 k allocations: 125.109 KiB)
      0.371618 seconds (4 allocations: 64 bytes)
      0.025982 seconds (8 allocations: 768 bytes)
    -1.5552639211087858e7
      0.276378 seconds (4 allocations: 64 bytes)
      0.000678 seconds (2.66 k allocations: 125.109 KiB)
      0.372626 seconds (4 allocations: 64 bytes)
      0.026436 seconds (8 allocations: 768 bytes)
    -1.5552639211087858e7
    -1.5552311626407882e7
    -1.5552328125458859e7
    Iteration 16: ll = -1.5552328125458859e7, reldiff = 2.000211184591851e-5
      1.756623 seconds (8.60 k allocations: 8.754 MiB)
    
    
    -1.5552328125458859e7
      0.277391 seconds (4 allocations: 64 bytes)
      0.000688 seconds (2.66 k allocations: 125.109 KiB)
      0.367031 seconds (4 allocations: 64 bytes)
      0.026239 seconds (8 allocations: 768 bytes)
    -1.5552328125458859e7
      0.279845 seconds (4 allocations: 64 bytes)
      0.000681 seconds (2.66 k allocations: 125.109 KiB)
      0.369247 seconds (4 allocations: 64 bytes)
      0.026166 seconds (8 allocations: 768 bytes)
    -1.5552328125458859e7
    -1.5552266154130427e7
    -1.5552259885576095e7
    Iteration 17: ll = -1.5552259885576095e7, reldiff = 4.387759968346335e-6
      1.772075 seconds (8.49 k allocations: 8.699 MiB, 0.94% gc time)
    
    
    -1.5552259885576095e7
      0.279333 seconds (4 allocations: 64 bytes)
      0.000689 seconds (2.66 k allocations: 125.109 KiB)
      0.367916 seconds (4 allocations: 64 bytes)
      0.026056 seconds (8 allocations: 768 bytes)
    -1.5552259885576095e7
      0.276034 seconds (4 allocations: 64 bytes)
      0.000681 seconds (2.66 k allocations: 125.109 KiB)
      0.370069 seconds (4 allocations: 64 bytes)
      0.026190 seconds (8 allocations: 768 bytes)
    -1.5552259885576095e7
    -1.5552235725528445e7
    -1.5552230063723285e7
    Iteration 18: ll = -1.5552230063723285e7, reldiff = 1.917525364808176e-6
      1.743838 seconds (8.49 k allocations: 8.697 MiB)
    
    
    -1.5552230063723285e7
      0.280163 seconds (4 allocations: 64 bytes)
      0.000682 seconds (2.66 k allocations: 125.109 KiB)
      0.367957 seconds (4 allocations: 64 bytes)
      0.026116 seconds (8 allocations: 768 bytes)
    -1.5552230063723285e7
      0.275350 seconds (4 allocations: 64 bytes)
      0.000582 seconds (2.66 k allocations: 125.109 KiB)
      0.371504 seconds (4 allocations: 64 bytes)
      0.026176 seconds (8 allocations: 768 bytes)
    -1.5552230063723285e7
    -1.555222164931152e7
    -1.5552216346354242e7
    Iteration 19: ll = -1.5552216346354242e7, reldiff = 8.820194265473032e-7
      1.840729 seconds (8.49 k allocations: 8.701 MiB)
    
    
    -1.5552216346354242e7
      0.280761 seconds (4 allocations: 64 bytes)
      0.000586 seconds (2.66 k allocations: 125.109 KiB)
      0.369551 seconds (4 allocations: 64 bytes)
      0.026236 seconds (8 allocations: 768 bytes)
    -1.5552216346354242e7
      0.277313 seconds (4 allocations: 64 bytes)
      0.000580 seconds (2.66 k allocations: 125.109 KiB)
      0.368692 seconds (4 allocations: 64 bytes)
      0.026159 seconds (8 allocations: 768 bytes)
    -1.5552216346354242e7
    -1.5552210406862404e7
    -1.5552207512250777e7
    Iteration 20: ll = -1.5552207512250777e7, reldiff = 5.680285863018589e-7
      1.746993 seconds (8.49 k allocations: 8.697 MiB)
    
    
    -1.5552207512250777e7
      0.277832 seconds (4 allocations: 64 bytes)
      0.000609 seconds (2.66 k allocations: 125.109 KiB)
      0.382785 seconds (4 allocations: 64 bytes)
      0.026582 seconds (8 allocations: 768 bytes)
    -1.5552207512250777e7
      0.296152 seconds (4 allocations: 64 bytes)
      0.000590 seconds (2.66 k allocations: 125.109 KiB)
      0.384742 seconds (4 allocations: 64 bytes)
      0.026081 seconds (8 allocations: 768 bytes)
    -1.5552207512250777e7
    -1.5552204951377708e7
    -1.5552200611477997e7
    Iteration 21: ll = -1.5552200611477997e7, reldiff = 4.437166090244169e-7
      1.793417 seconds (8.49 k allocations: 8.697 MiB)
    
    
    -1.5552200611477997e7
      0.278324 seconds (4 allocations: 64 bytes)
      0.000580 seconds (2.66 k allocations: 125.109 KiB)
      0.369446 seconds (4 allocations: 64 bytes)
      0.025987 seconds (8 allocations: 768 bytes)
    -1.5552200611477997e7
      0.277572 seconds (4 allocations: 64 bytes)
      0.000581 seconds (2.66 k allocations: 125.109 KiB)
      0.399638 seconds (4 allocations: 64 bytes)
      0.026015 seconds (8 allocations: 768 bytes)
    -1.5552200611477997e7
    -1.5552198775938418e7
    -1.5552197336867832e7
    Iteration 22: ll = -1.5552197336867832e7, reldiff = 2.1055606517124269e-7
      1.777111 seconds (8.49 k allocations: 8.697 MiB)
    
    
    -1.5552197336867832e7
      0.278276 seconds (4 allocations: 64 bytes)
      0.000583 seconds (2.66 k allocations: 125.109 KiB)
      0.369680 seconds (4 allocations: 64 bytes)
      0.026021 seconds (8 allocations: 768 bytes)
    -1.5552197336867832e7
      0.282055 seconds (4 allocations: 64 bytes)
      0.000581 seconds (2.66 k allocations: 125.109 KiB)
      0.369828 seconds (4 allocations: 64 bytes)
      0.026026 seconds (8 allocations: 768 bytes)
    -1.5552197336867832e7
    -1.5552196340740334e7
    -1.5552194868097581e7
    Iteration 23: ll = -1.5552194868097581e7, reldiff = 1.587409288349003e-7
      1.750541 seconds (8.49 k allocations: 8.705 MiB)
    
    
    -1.5552194868097581e7
      0.286798 seconds (4 allocations: 64 bytes)
      0.000606 seconds (2.66 k allocations: 125.109 KiB)
      0.369022 seconds (4 allocations: 64 bytes)
      0.025984 seconds (8 allocations: 768 bytes)
    -1.5552194868097581e7
      0.276643 seconds (4 allocations: 64 bytes)
      0.000578 seconds (2.66 k allocations: 125.109 KiB)
      0.371678 seconds (4 allocations: 64 bytes)
      0.026030 seconds (8 allocations: 768 bytes)
    -1.5552194868097581e7
    -1.5552194288014418e7
    -1.5552193929721253e7
    Iteration 24: ll = -1.5552193929721253e7, reldiff = 6.033722802055984e-8
     55.143828 seconds (24.93 M allocations: 1.403 GiB, 1.45% gc time, 22.49% compilation time)





    (AdmixData{Float64}(379, 54051, 4, 3, true, [0.07097194246589222 0.03448150055692116 … 0.99999 0.9903852642383892; 0.41754050336181514 0.7462395931384954 … 0.9939184541764243 0.9207298826668833; 1.0e-5 0.21926890630458337 … 0.9817535223439229 0.99999; 0.5114775541722926 1.0e-5 … 0.9692260172250666 0.99999], [0.07075723653349945 0.03486513012092069 … 0.99999 0.9902814082751056; 0.41762319843677725 0.7438524827975902 … 0.9939080061760321 0.9206403753110268; 1.0e-5 0.2212723870814891 … 0.981798541025298 0.99999; 0.5116095650297232 1.0e-5 … 0.9692097179053448 0.99999], [0.0707761786648089 0.034789786470223884 … 0.99999 0.9903072106563765; 0.41760673781774055 0.7444353552668401 … 0.9939107258213955 0.9206600600274841; 1.0e-5 0.22076485826293593 … 0.9817883444261288 0.99999; 0.5116070835174504 1.0e-5 … 0.9692129945070692 0.99999], [0.07097194246589222 0.03448150055692116 … 0.99999 0.9903852642383892; 0.41754050336181514 0.7462395931384954 … 0.9939184541764243 0.9207298826668833; 1.0e-5 0.21926890630458337 … 0.9817535223439229 0.99999; 0.5114775541722926 1.0e-5 … 0.9692260172250666 0.99999], [0.07097194246589222, 0.41754050336181514, 1.0e-5, 0.5114775541722926, 0.03448150055692116, 0.7462395931384954, 0.21926890630458337, 1.0e-5, 0.08018092101420532, 0.7144266384362248  …  0.9479365586807617, 0.9685993514983139, 0.99999, 0.9939184541764243, 0.9817535223439229, 0.9692260172250666, 0.9903852642383892, 0.9207298826668833, 0.99999, 0.99999], [0.07075723653349945, 0.41762319843677725, 1.0e-5, 0.5116095650297232, 0.03486513012092069, 0.7438524827975902, 0.2212723870814891, 1.0e-5, 0.08017037747345482, 0.7128118897135658  …  0.9479592824854917, 0.9686421564528299, 0.99999, 0.9939080061760321, 0.981798541025298, 0.9692097179053448, 0.9902814082751056, 0.9206403753110268, 0.99999, 0.99999], [0.0707761786648089, 0.41760673781774055, 1.0e-5, 0.5116070835174504, 0.034789786470223884, 0.7444353552668401, 0.22076485826293593, 1.0e-5, 0.08028380357696328, 0.7130294409704403  …  0.9479524409541661, 0.9686249917676314, 0.99999, 0.9939107258213955, 0.9817883444261288, 0.9692129945070692, 0.9903072106563765, 0.9206600600274841, 0.99999, 0.99999], [0.07097194246589222, 0.41754050336181514, 1.0e-5, 0.5114775541722926, 0.03448150055692116, 0.7462395931384954, 0.21926890630458337, 1.0e-5, 0.08018092101420532, 0.7144266384362248  …  0.9479365586807617, 0.9685993514983139, 0.99999, 0.9939184541764243, 0.9817535223439229, 0.9692260172250666, 0.9903852642383892, 0.9207298826668833, 0.99999, 0.99999], [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0  …  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0  …  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0], [0.07097194246589222 0.03448150055692116 … 1.0e-5 1.0e-5; 0.41754050336181514 0.7462395931384954 … 1.0e-5 1.0e-5; 1.0e-5 0.21926890630458337 … 1.0e-5 1.0e-5; 0.5114775541722926 1.0e-5 … 0.9999699999999999 0.9999699999999999], [0.07075723653349945 0.03486513012092069 … 1.0e-5 1.0e-5; 0.41762319843677725 0.7438524827975902 … 1.0e-5 1.0e-5; 1.0e-5 0.2212723870814891 … 1.0e-5 1.0e-5; 0.5116095650297232 1.0e-5 … 0.9999699999999999 0.9999699999999999], [0.0707761786648089 0.034789786470223884 … 1.0e-5 1.0e-5; 0.41760673781774055 0.7444353552668401 … 1.0e-5 1.0e-5; 1.0e-5 0.22076485826293593 … 1.0e-5 1.0e-5; 0.5116070835174504 1.0e-5 … 0.9999699999999999 0.9999699999999999], [0.07097194246589222 0.03448150055692116 … 1.0e-5 1.0e-5; 0.41754050336181514 0.7462395931384954 … 1.0e-5 1.0e-5; 1.0e-5 0.21926890630458337 … 1.0e-5 1.0e-5; 0.5114775541722926 1.0e-5 … 0.9999699999999999 0.9999699999999999], [0.8417036781784455 0.9719130016593888 … 0.99999 0.9903852642383892; 0.9235910956222408 0.9920924958274663 … 0.9939184541764243 0.9207298826668833; 0.91779284272664 0.9829380093686094 … 0.9817535223439229 0.99999; 0.9129060765064565 0.9997632898448726 … 0.9692260172250666 0.99999], [0.8417308884182984 0.9719333669762293 … 0.99999 0.9902814082751056; 0.9237319202056826 0.9921163128465725 … 0.9939080061760321 0.9206403753110268; 0.9176758088904347 0.9829477937654668 … 0.981798541025298 0.99999; 0.9128799686593018 0.9997248131649172 … 0.9692097179053448 0.99999], [0.841690966855962 0.9719287572433678 … 0.99999 0.9903072106563765; 0.9236789074293282 0.9921107074985339 … 0.9939107258213955 0.9206600600274841; 0.9177364888509721 0.9829464228879098 … 0.9817883444261288 0.99999; 0.9129052987772184 0.9997319203980494 … 0.9692129945070692 0.99999], [0.8417036781784455 0.9719130016593888 … 0.99999 0.9903852642383892; 0.9235910956222408 0.9920924958274663 … 0.9939184541764243 0.9207298826668833; 0.91779284272664 0.9829380093686094 … 0.9817535223439229 0.99999; 0.9129060765064565 0.9997632898448726 … 0.9692260172250666 0.99999], [112429.32359917002 108324.98954890843 108500.55874838612 107321.65837279192; 108324.98954890843 109873.97275634346 107801.78465825132 106624.68476500905; 108500.55874838612 107801.78465825132 111230.48889015679 108062.85088666201; 107321.65837279192 106624.68476500905 108062.85088666201 109415.85315484426;;; 113156.90784154511 107685.88282075372 108700.2019605354 108747.49005223403; 107685.88282075372 108436.25957312505 107047.39093836695 107098.21700861708; 108700.2019605354 107047.39093836695 111542.06205844568 109548.63620848597; 108747.49005223403 107098.21700861708 109548.63620848597 111208.83072976909;;; 112308.03142412317 107517.24558495541 108487.70293022148 108716.68538726468; 107517.24558495541 108471.14732477836 107059.13756936864 107220.56178693537; 108487.7029302215 107059.13756936864 111537.15613171147 109478.71640831263; 108716.68538726468 107220.56178693537 109478.71640831263 111355.75193228382;;; … ;;; 113611.68931612682 110080.65040250416 107257.70045606881 108707.17677049637; 110080.65040250416 112995.51987769718 107411.64877388586 108763.28793921293; 107257.70045606881 107411.64877388586 108267.21786953261 107555.8197428564; 108707.17677049637 108763.28793921293 107555.81974285639 110739.47079430547;;; 112904.84798858447 109852.2892163234 108807.81333412504 107518.5263835833; 109852.2892163234 111648.01373136841 108742.1737792436 107454.05957415157; 108807.81333412504 108742.1737792436 110905.85334016576 107791.91612136445; 107518.5263835833 107454.05957415157 107791.91612136445 108102.03082914864;;; 114151.93577995931 110844.7316121083 109990.83498763376 107612.39773994582; 110844.7316121083 112764.8322387724 109840.57607247417 107574.54134033063; 109990.83498763376 109840.57607247417 112402.00967340921 108064.02014068769; 107612.39773994582 107574.54134033063 108064.02014068769 108102.02109867314], [-108102.08881321369 -108101.08225339273 … -107518.61647729724 -107612.51924303739; -108101.98344736466 -108102.76593143288 … -107454.13837713147 -107574.64860548984; -107984.82624843404 -108099.5880300587 … -107791.96692228544 -108064.1005542907; -108102.00351894205 -107697.97033973054 … -108102.01541324455 -108102.01054763237], [924.5741271099636 227.25240605343262 160.09517945686386 115.71526330876449; 227.25240605343257 1732.512599932291 688.5541017787021 339.6747249587286; 160.09517945686386 688.5541017787021 1570.707478695452 267.6595248475699; 115.71526330876449 339.6747249587286 267.6595248475699 1178.0746757891968;;; 4768.153448329896 2586.3242978235135 628.6298851193931 2104.539504392266; 2586.3242978235135 14152.812368293296 2864.630742804071 5534.830341543266; 628.6298851193931 2864.630742804071 10107.367856338271 2885.235009219228; 2104.539504392266 5534.830341543266 2885.235009219228 10558.203131915776;;; 6279.959099555947 998.4064986577184 472.30159200491687 489.69489152367606; 998.4064986577184 2689.138852356804 1077.9498881189143 453.11301014643175; 472.30159200491687 1077.9498881189145 2767.0784548892957 589.8876790539748; 489.69489152367606 453.11301014643175 589.8876790539748 1678.7805484618664;;; … ;;; 1631.311460636115 336.15706054385527 302.53764718806946 281.01944344319566; 336.1570605438553 2240.600708663522 1062.3874895759602 1129.0304343233577; 302.53764718806946 1062.3874895759602 2359.20908542582 452.0190754402239; 281.01944344319566 1129.0304343233577 452.0190754402239 1585.943164837384;;; 9262.734784434671 192.32792457293561 640.2147875336885 4281.532621735132; 192.32792457293561 29718.58058305707 2938.956083808823 34.91871865568121; 640.2147875336884 2938.956083808823 8550.450794983517 1294.7924247800922; 4281.532621735132 34.91871865568121 1294.7924247800922 4353.246261607954;;; 5275.579066366831 1404.022476678692 144.8544174690879 84.65866849531672; 1404.022476678692 2893.8569447678096 1350.0814380376473 1137.6907137204062; 144.8544174690879 1350.0814380376473 2426.02875785844 645.1324357344375; 84.65866849531672 1137.6907137204062 645.1324357344374 1338.7204582833865], [0.03631207419205951 0.022381483177531747 … -16.52354541323669 -0.16577538547253035; 0.050532137525475626 0.05584343156612803 … -0.05289433670461863 -0.11545962378720276; -0.05919690085704499 0.012304994058437657 … 0.06854793421832994 -107.37909057514909; -0.02345570553259435 -0.03035829188345396 … -0.04397166759326865 -65.5267184035896], SubArray{Float64, 1, Matrix{Float64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[0.07097194246589222, 0.41754050336181514, 1.0e-5, 0.5114775541722926], [0.03448150055692116, 0.7462395931384954, 0.21926890630458337, 1.0e-5], [0.08018092101420532, 0.7144266384362248, 0.20538244054956978, 1.0e-5], [0.11632077555134018, 0.36853675824615245, 1.0000000000055512e-5, 0.5151324662025073], [1.0e-5, 0.974944905318529, 0.025035094681470978, 1.0e-5], [0.005807777302670947, 0.856662436189591, 0.137519786507738, 1.0000000000055512e-5], [1.0e-5, 0.6329044385157624, 0.3670755614842376, 1.0e-5], [0.17770918465002392, 0.6115257239486562, 0.2107550914013197, 1.0e-5], [0.05705494573765796, 0.5054313809923692, 1.0e-5, 0.4375036732699729], [0.026642645032760592, 0.935823075121205, 0.03752427984603438, 1.0000000000055512e-5]  …  [1.0e-5, 1.0e-5, 1.0e-5, 0.9999699999999999], [1.0e-5, 1.0e-5, 0.3967512591287673, 0.6032287408712326], [0.02454209264096739, 0.2013724991853736, 0.7740754081736589, 1.0e-5], [1.0e-5, 1.0e-5, 1.0e-5, 0.9999699999999999], [1.0000000000055512e-5, 0.2084490281708426, 0.7915309718291573, 1.0000000000055512e-5], [0.04935553945217813, 1.0e-5, 0.5341517884758102, 0.4164826720720115], [1.0e-5, 0.19909881536078572, 0.4688920647547747, 0.33199911988443964], [0.09811802806306286, 0.07930050718129622, 0.8225714647556408, 1.0000000000083268e-5], [1.0e-5, 1.0e-5, 1.0e-5, 0.9999699999999999], [1.0e-5, 1.0e-5, 1.0e-5, 0.9999699999999999]], SubArray{Float64, 1, Matrix{Float64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[0.07075723653349945, 0.41762319843677725, 1.0e-5, 0.5116095650297232], [0.03486513012092069, 0.7438524827975902, 0.2212723870814891, 1.0e-5], [0.08017037747345482, 0.7128118897135658, 0.20700773281297916, 1.0e-5], [0.11598873061362942, 0.3688546567006869, 1.0e-5, 0.5151466126856836], [1.0e-5, 0.9713631129429602, 0.028616887057039804, 1.0e-5], [0.006068636897630073, 0.854336837500445, 0.13958452560192494, 1.0e-5], [1.0e-5, 0.631190662255502, 0.3687893377444979, 1.0e-5], [0.17812156197868603, 0.6095970194611892, 0.21227141856012466, 1.0e-5], [0.05714253180678263, 0.5047449621694039, 1.0e-5, 0.4381025060238134], [0.02883022923620386, 0.9286854208184038, 0.04247434994539219, 1.0e-5]  …  [1.0e-5, 1.0e-5, 1.0e-5, 0.9999699999999999], [1.0e-5, 1.0e-5, 0.3966804462769348, 0.6032995537230652], [0.024850351560056876, 0.19994871739243403, 0.775190931047509, 1.0e-5], [1.0e-5, 1.0e-5, 1.0e-5, 0.9999699999999999], [1.0e-5, 0.20759714100354, 0.7923828589964599, 1.0e-5], [0.04937874968182977, 1.0e-5, 0.5341343821799872, 0.416476868138183], [1.0e-5, 0.19883802393724676, 0.4691310311769159, 0.3320209448858374], [0.0987833535268654, 0.07699509686296713, 0.8242115496101674, 1.0e-5], [1.0e-5, 1.0e-5, 1.0e-5, 0.9999699999999999], [1.0e-5, 1.0e-5, 1.0e-5, 0.9999699999999999]], SubArray{Float64, 1, Matrix{Float64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[0.07097194246589222, 0.41754050336181514, 1.0e-5, 0.5114775541722926], [0.03448150055692116, 0.7462395931384954, 0.21926890630458337, 1.0e-5], [0.08018092101420532, 0.7144266384362248, 0.20538244054956978, 1.0e-5], [0.11632077555134018, 0.36853675824615245, 1.0000000000055512e-5, 0.5151324662025073], [1.0e-5, 0.974944905318529, 0.025035094681470978, 1.0e-5], [0.005807777302670947, 0.856662436189591, 0.137519786507738, 1.0000000000055512e-5], [1.0e-5, 0.6329044385157624, 0.3670755614842376, 1.0e-5], [0.17770918465002392, 0.6115257239486562, 0.2107550914013197, 1.0e-5], [0.05705494573765796, 0.5054313809923692, 1.0e-5, 0.4375036732699729], [0.026642645032760592, 0.935823075121205, 0.03752427984603438, 1.0000000000055512e-5]  …  [1.0e-5, 1.0e-5, 1.0e-5, 0.9999699999999999], [1.0e-5, 1.0e-5, 0.3967512591287673, 0.6032287408712326], [0.02454209264096739, 0.2013724991853736, 0.7740754081736589, 1.0e-5], [1.0e-5, 1.0e-5, 1.0e-5, 0.9999699999999999], [1.0000000000055512e-5, 0.2084490281708426, 0.7915309718291573, 1.0000000000055512e-5], [0.04935553945217813, 1.0e-5, 0.5341517884758102, 0.4164826720720115], [1.0e-5, 0.19909881536078572, 0.4688920647547747, 0.33199911988443964], [0.09811802806306286, 0.07930050718129622, 0.8225714647556408, 1.0000000000083268e-5], [1.0e-5, 1.0e-5, 1.0e-5, 0.9999699999999999], [1.0e-5, 1.0e-5, 1.0e-5, 0.9999699999999999]], SubArray{Float64, 1, Matrix{Float64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[0.8417036781784455, 0.9235910956222408, 0.91779284272664, 0.9129060765064565], [0.9719130016593888, 0.9920924958274663, 0.9829380093686094, 0.9997632898448726], [0.9915180800008472, 0.9392051926830348, 0.9593727624537517, 0.935531397359741], [0.5460568839367185, 0.47168935139816404, 0.5656330896981703, 0.45669235846612516], [0.7261583819137465, 0.691390905524685, 0.666198216932041, 0.6239379823851668], [0.8565411071858399, 0.9205265143270079, 0.9164140274699304, 0.9364270052423508], [0.6595924061529945, 0.6528518334406135, 0.6365491200564127, 0.5391835503089367], [0.9579078014880861, 0.9373407852283834, 0.9590424848836159, 0.9348729848753671], [0.9592145462442412, 0.9279632971710615, 0.9562201778657994, 0.9385245880678953], [0.8327041355065217, 0.9048774985294098, 0.9010071125939634, 0.8956880541752188]  …  [0.6756704584235717, 0.7314549617306517, 0.8340600114648455, 0.7513361730841577], [0.746784293163343, 0.691617785276156, 0.9007057137392552, 0.8208805519516201], [0.8115350059973747, 0.8586501627604565, 0.9391172968658348, 0.8775937175036945], [0.840948246262924, 0.800505282567632, 0.8852047370256159, 0.9780599290375627], [0.9356703995236214, 0.866093814271617, 0.9314922044160869, 0.99999], [0.9251428631025986, 0.8555205017588279, 0.9367611058889435, 0.99999], [0.851459893740546, 0.7789118270663608, 0.8798011776415458, 0.9880406588401981], [0.9201564359347043, 0.9427045029667565, 0.9479365586807617, 0.9685993514983139], [0.99999, 0.9939184541764243, 0.9817535223439229, 0.9692260172250666], [0.9903852642383892, 0.9207298826668833, 0.99999, 0.99999]], SubArray{Float64, 1, Matrix{Float64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[0.8417308884182984, 0.9237319202056826, 0.9176758088904347, 0.9128799686593018], [0.9719333669762293, 0.9921163128465725, 0.9829477937654668, 0.9997248131649172], [0.9915378028060944, 0.9391058490331118, 0.9593574051503967, 0.9356052120220393], [0.5460666204954089, 0.47155804092937803, 0.565508067879408, 0.4567741765774998], [0.7262142717138594, 0.6912287188179169, 0.6662600119917078, 0.624090119345088], [0.8565783393796349, 0.9205736021876723, 0.9163574980741109, 0.9364346903358332], [0.6596255930906725, 0.6527885747776728, 0.6365353109878313, 0.5393069104772885], [0.957884064224064, 0.937350345287009, 0.9590070022595227, 0.9348661885955184], [0.9591862111878382, 0.9279692192990981, 0.9561730294782625, 0.9385217770001347], [0.8327590788222605, 0.9049088448357865, 0.900922473418964, 0.8957438058281848]  …  [0.6756732049422645, 0.7313177183047539, 0.8340165589155497, 0.7513526598164667], [0.7468481276884283, 0.6912146548167456, 0.900595911237113, 0.8209839218356867], [0.8116160570015214, 0.8584870823670131, 0.9390497280468919, 0.8776512059643605], [0.8409705342939152, 0.8002549962754852, 0.8851973308760203, 0.9781811183941158], [0.9356595335105431, 0.8659892987794476, 0.9314625058198851, 0.99999], [0.9251380776944745, 0.8553966251733212, 0.9367172842854734, 0.99999], [0.8514854440000209, 0.7786587539071353, 0.8797496883906782, 0.9881643094013802], [0.9201776168035013, 0.9426370315872961, 0.9479592824854917, 0.9686421564528299], [0.99999, 0.9939080061760321, 0.981798541025298, 0.9692097179053448], [0.9902814082751056, 0.9206403753110268, 0.99999, 0.99999]], SubArray{Float64, 1, Matrix{Float64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[0.8417036781784455, 0.9235910956222408, 0.91779284272664, 0.9129060765064565], [0.9719130016593888, 0.9920924958274663, 0.9829380093686094, 0.9997632898448726], [0.9915180800008472, 0.9392051926830348, 0.9593727624537517, 0.935531397359741], [0.5460568839367185, 0.47168935139816404, 0.5656330896981703, 0.45669235846612516], [0.7261583819137465, 0.691390905524685, 0.666198216932041, 0.6239379823851668], [0.8565411071858399, 0.9205265143270079, 0.9164140274699304, 0.9364270052423508], [0.6595924061529945, 0.6528518334406135, 0.6365491200564127, 0.5391835503089367], [0.9579078014880861, 0.9373407852283834, 0.9590424848836159, 0.9348729848753671], [0.9592145462442412, 0.9279632971710615, 0.9562201778657994, 0.9385245880678953], [0.8327041355065217, 0.9048774985294098, 0.9010071125939634, 0.8956880541752188]  …  [0.6756704584235717, 0.7314549617306517, 0.8340600114648455, 0.7513361730841577], [0.746784293163343, 0.691617785276156, 0.9007057137392552, 0.8208805519516201], [0.8115350059973747, 0.8586501627604565, 0.9391172968658348, 0.8775937175036945], [0.840948246262924, 0.800505282567632, 0.8852047370256159, 0.9780599290375627], [0.9356703995236214, 0.866093814271617, 0.9314922044160869, 0.99999], [0.9251428631025986, 0.8555205017588279, 0.9367611058889435, 0.99999], [0.851459893740546, 0.7789118270663608, 0.8798011776415458, 0.9880406588401981], [0.9201564359347043, 0.9427045029667565, 0.9479365586807617, 0.9685993514983139], [0.99999, 0.9939184541764243, 0.9817535223439229, 0.9692260172250666], [0.9903852642383892, 0.9207298826668833, 0.99999, 0.99999]], SubArray{Float64, 2, Array{Float64, 3}, Tuple{Base.Slice{Base.OneTo{Int64}}, Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[112429.32359917002 108324.98954890843 108500.55874838612 107321.65837279192; 108324.98954890843 109873.97275634346 107801.78465825132 106624.68476500905; 108500.55874838612 107801.78465825132 111230.48889015679 108062.85088666201; 107321.65837279192 106624.68476500905 108062.85088666201 109415.85315484426], [113156.90784154511 107685.88282075372 108700.2019605354 108747.49005223403; 107685.88282075372 108436.25957312505 107047.39093836695 107098.21700861708; 108700.2019605354 107047.39093836695 111542.06205844568 109548.63620848597; 108747.49005223403 107098.21700861708 109548.63620848597 111208.83072976909], [112308.03142412317 107517.24558495541 108487.70293022148 108716.68538726468; 107517.24558495541 108471.14732477836 107059.13756936864 107220.56178693537; 108487.7029302215 107059.13756936864 111537.15613171147 109478.71640831263; 108716.68538726468 107220.56178693537 109478.71640831263 111355.75193228382], [112239.46671552368 108242.06907723553 108550.08009652735 107070.56746775005; 108242.06907723552 109953.22243309986 107856.87186511658 106744.47710962991; 108550.08009652735 107856.87186511658 111392.42406952858 108077.08567763619; 107070.56746775005 106744.47710962991 108077.08567763619 109306.49146875324], [113106.42060693474 107855.8479790398 109649.35517063465 109354.36295709205; 107855.8479790398 108106.9895734995 107938.04115721623 107606.17802904946; 109649.35517063465 107938.04115721623 113498.34008650856 111046.13742248678; 109354.36295709205 107606.17802904946 111046.13742248678 112166.95961757172], [112985.3688008684 107839.57677823436 109486.74542535232 109698.12871292105; 107839.57677823436 108224.64199896785 107366.56411450148 107620.12343758611; 109486.74542535232 107366.56411450148 112520.33482003838 110453.84835244845; 109698.12871292105 107620.12343758611 110453.84835244845 129833.56079129408], [112500.0252252583 108058.33153839948 108037.86922013585 108394.7487783574; 108058.33153839948 108839.43335672998 106842.22644847479 107300.72320930884; 108037.86922013585 106842.22644847479 110254.13841129624 108741.57273523886; 108394.74877835742 107300.72320930884 108741.57273523886 111051.82492157437], [111111.91314175083 107287.57887795389 107913.05041311138 108060.2623950258; 107287.57887795389 108685.30808919617 107113.60013253857 107257.86156625516; 107913.05041311138 107113.60013253857 111091.31909002071 109397.54174970483; 108060.2623950258 107257.86156625516 109397.54174970483 111435.56322715625], [112462.18985731964 108212.6137940057 108748.22052833323 107405.95187965578; 108212.6137940057 109337.1273311108 107638.3838486113 106665.49898055702; 108748.22052833323 107638.3838486113 112011.63179487186 108482.37134474129; 107405.95187965578 106665.49898055702 108482.37134474129 109846.7138936029], [113601.27010983805 107849.70267820818 109759.21993710937 109448.75429964192; 107849.70267820818 108124.03089502535 107804.19564437009 107531.91572238208; 109759.21993710937 107804.19564437009 113302.04721723696 110912.46870600317; 109448.75429964192 107531.91572238208 110912.46870600317 112260.76899609643]  …  [113797.6686591533 110344.82072796888 109018.26735377399 107574.18681414625; 110344.82072796891 112231.48951242777 108940.81615440211 107548.72133117607; 109018.26735377399 108940.81615440211 111115.8271952366 107879.25544822127; 107574.18681414625 107548.72133117607 107879.25544822127 108102.02607542684], [114360.35812794034 110622.28099352628 108531.98256567089 107785.83096222022; 110622.28099352628 113089.08907724683 108491.42747498136 107644.1849635769; 108531.98256567089 108491.42747498136 109748.56145933064 107019.43786092225; 107785.83096222022 107644.1849635769 107019.43786092225 108813.75819344888], [115432.31722747313 110158.06761393118 107337.47732338728 108982.55267993452; 110158.06761393118 111611.6726574322 107132.42520862664 108497.60455282792; 107337.47732338728 107132.42520862664 108376.16561354336 107610.7717238049; 108982.55267993452 108497.60455282792 107610.7717238049 110799.8458217023], [113526.55451490029 110564.99050444039 109162.4306444606 107765.61604330955; 110564.99050444039 112637.69868627921 109098.39611370387 107787.74314891658; 109162.4306444606 109098.39611370387 111039.20220254558 107752.45775518261; 107765.61604330955 107787.74314891658 107752.45775518261 108102.0200020675], [114003.4215749945 109706.9062625 107286.55401906556 108386.84415451944; 109706.9062625 111649.1057775992 107174.25220959245 108375.5199017655; 107286.55401906556 107174.25220959245 108344.67249786768 107626.67994724773; 108386.84415451944 108375.5199017655 107626.67994724773 112125.7300429555], [113388.8803411852 110005.83237210888 107775.07197910821 107893.75553967796; 110005.83237210888 112399.5501116194 107946.11110865443 107874.94765921326; 107775.07197910821 107946.11110865443 108966.50406561652 107032.11120618941; 107893.75553967796 107874.94765921326 107032.11120618941 109498.81766728371], [112377.10264117553 109046.560495385 107447.82002888172 107403.8019881487; 109046.560495385 111192.7959992327 107369.78644277585 107286.64214744967; 107447.82002888172 107369.78644277585 109067.07679814361 107176.3286535458; 107403.8019881487 107286.64214744967 107176.3286535458 109898.45881279373], [113611.68931612682 110080.65040250416 107257.70045606881 108707.17677049637; 110080.65040250416 112995.51987769718 107411.64877388586 108763.28793921293; 107257.70045606881 107411.64877388586 108267.21786953261 107555.8197428564; 108707.17677049637 108763.28793921293 107555.81974285639 110739.47079430547], [112904.84798858447 109852.2892163234 108807.81333412504 107518.5263835833; 109852.2892163234 111648.01373136841 108742.1737792436 107454.05957415157; 108807.81333412504 108742.1737792436 110905.85334016576 107791.91612136445; 107518.5263835833 107454.05957415157 107791.91612136445 108102.03082914864], [114151.93577995931 110844.7316121083 109990.83498763376 107612.39773994582; 110844.7316121083 112764.8322387724 109840.57607247417 107574.54134033063; 109990.83498763376 109840.57607247417 112402.00967340921 108064.02014068769; 107612.39773994582 107574.54134033063 108064.02014068769 108102.02109867314]], SubArray{Float64, 1, Matrix{Float64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[-108102.08881321369, -108101.98344736466, -107984.82624843404, -108102.00351894205], [-108101.08225339273, -108102.76593143288, -108099.5880300587, -107697.97033973054], [-108102.22886330938, -108102.36321819141, -108100.67485946715, -107808.00340113461], [-108102.23014928652, -108101.75267960558, -108050.75396303699, -108102.1262611736], [-107907.24006265373, -108102.14727622095, -108097.20779778186, -107704.68204815735], [-108100.74256128413, -108102.52473834292, -108098.84823687006, -108028.5003793744], [-108050.83303470758, -108102.86154557625, -108100.53415157701, -107832.14160303895], [-108101.55272742211, -108102.6994268794, -108100.3783503928, -107855.02118003159], [-108102.05007924567, -108102.40571757284, -108071.59949596935, -108101.52672822447], [-108096.64318052726, -108102.53123413575, -108094.10819727638, -107730.81269694115]  …  [-107574.2911961091, -107548.81004080015, -107879.30981966468, -108102.01303628046], [-108081.90882303941, -107980.35373110719, -108102.0576732268, -108101.96442817365], [-108102.6268024637, -108103.24692015903, -108101.66189170057, -107822.21394027401], [-107765.71561458506, -107787.832527475, -107752.51818173948, -108102.00999963666], [-107789.09039660082, -108103.25635593763, -108101.6788311389, -107782.1895730179], [-108101.72608219048, -108018.22417317769, -108102.03255508128, -108101.99273594472], [-107751.14480537978, -108102.35722871934, -108101.80675975636, -108102.06967299628], [-108102.73658326604, -108105.24591822934, -108101.61261461335, -107762.55561840134], [-107518.61647729724, -107454.13837713147, -107791.96692228544, -108102.01541324455], [-107612.51924303739, -107574.64860548984, -108064.1005542907, -108102.01054763237]], SubArray{Float64, 2, Array{Float64, 3}, Tuple{Base.Slice{Base.OneTo{Int64}}, Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[924.5741271099636 227.25240605343262 160.09517945686386 115.71526330876449; 227.25240605343257 1732.512599932291 688.5541017787021 339.6747249587286; 160.09517945686386 688.5541017787021 1570.707478695452 267.6595248475699; 115.71526330876449 339.6747249587286 267.6595248475699 1178.0746757891968], [4768.153448329896 2586.3242978235135 628.6298851193931 2104.539504392266; 2586.3242978235135 14152.812368293296 2864.630742804071 5534.830341543266; 628.6298851193931 2864.630742804071 10107.367856338271 2885.235009219228; 2104.539504392266 5534.830341543266 2885.235009219228 10558.203131915776], [6279.959099555947 998.4064986577184 472.30159200491687 489.69489152367606; 998.4064986577184 2689.138852356804 1077.9498881189143 453.11301014643175; 472.30159200491687 1077.9498881189145 2767.0784548892957 589.8876790539748; 489.69489152367606 453.11301014643175 589.8876790539748 1678.7805484618664], [459.24672772190337 87.9738666243315 61.31969423598148 40.5786154945309; 87.9738666243315 510.37971827723754 197.86987536704245 133.80417321596596; 61.31969423598147 197.86987536704245 499.0507518978236 91.38413729900438; 40.5786154945309 133.80417321596596 91.38413729900438 350.39540199131187], [563.8964748434147 104.30557882739748 68.25660110567912 48.726885144557805; 104.30557882739748 595.1445719744926 225.0158994126817 144.94274587164233; 68.25660110567912 225.01589941268165 566.4796694328207 95.85726948044041; 48.726885144557805 144.94274587164233 95.85726948044041 389.5202388295697], [988.6733270446975 257.9468709323089 162.3122866400809 140.37766939995745; 257.9468709323089 1681.3527758432103 644.8051156176427 452.2160648087685; 162.3122866400809 644.8051156176426 1692.8692326863206 215.35996627816206; 140.37766939995745 452.2160648087685 215.35996627816203 1456.7643702186033], [500.40731732570276 96.98430903972941 66.64213583856144 43.83260869026991; 96.98430903972941 551.9816380189432 214.95946998065668 138.53167043756687; 66.64213583856144 214.95946998065668 534.6180087409708 92.50271134576334; 43.83260869026991 138.53167043756687 92.50271134576334 361.9764682940456], [2706.3964013435575 412.7462271027425 455.3847498869713 165.36968429554145; 412.7462271027425 2378.8966025310274 1071.6650134015902 547.6619764912397; 455.3847498869713 1071.6650134015902 2584.4304306495605 441.67647336144046; 165.36968429554148 547.6619764912397 441.6764733614404 1604.6988082838138], [2763.2517256808783 377.0687765999483 434.9711310980846 168.69860851682614; 377.0687765999483 2176.0476486301586 1010.5430463344144 503.4598375150728; 434.97113109808447 1010.5430463344144 2380.333624817995 436.7798767018105; 168.69860851682614 503.4598375150728 436.7798767018105 1654.4798961255965], [853.6167961839351 226.3368418140697 133.664716768385 117.48816721948931; 226.3368418140697 1413.8321671371964 561.5618823115217 296.54305321283255; 133.664716768385 561.5618823115217 1371.349941152869 223.74901883228577; 117.48816721948933 296.54305321283255 223.74901883228577 978.1917507118118]  …  [533.414850451095 110.16386583991769 87.69546360602304 54.76479661808685; 110.16386583991769 685.6470763809817 274.57751720086856 175.04724269318038; 87.69546360602304 274.57751720086856 773.65496093058 143.55708465231322; 54.76479661808685 175.04724269318038 143.55708465231322 467.4432950329113], [605.7347588289492 117.85327287727469 110.16406149424112 78.0780989302318; 117.85327287727469 695.0211558109091 288.1757004863465 175.38239702134433; 110.16406149424114 288.1757004863465 1029.8981253137217 191.2204681992634; 78.07809893023179 175.38239702134433 191.2204681992634 561.8121870451746], [767.3530685170407 174.04777741050853 165.57819225140977 126.39581990771471; 174.04777741050853 1178.6275253066021 498.5903749760059 292.1509051903901; 165.57819225140977 498.5903749760059 1638.037383790167 251.36232843613382; 126.39581990771471 292.1509051903901 251.36232843613382 772.6129470958479], [902.815736573503 156.08648815443775 118.59443769914365 106.80855118015243; 156.08648815443775 1003.2306549851588 366.37254809346405 443.24026236556716; 118.59443769914365 366.3725480934641 1242.215338265734 365.5086925143014; 106.80855118015245 443.24026236556716 365.5086925143014 793.5666273692071], [1994.8401631250113 249.39907110162815 170.83696185361921 116.19562173224263; 249.39907110162815 1516.7401371056997 549.0348434709191 604.4022563202761; 170.83696185361921 549.0348434709191 2102.2800852077035 860.0868707985828; 116.19562173224263 604.4022563202761 860.0868707985828 1454.3495130897697], [1744.5498117256263 235.4850037719598 160.91234186045597 92.01781450941424; 235.4850037719598 1427.7981907112016 540.2217328711631 521.4745245422888; 160.91234186045597 540.2217328711631 2189.9423563819746 992.4328384842911; 92.01781450941424 521.4745245422888 992.4328384842911 1492.09435819795], [949.7819758486739 148.53155080672877 122.54568287977382 115.31526599245463; 148.53155080672875 944.8883686727771 371.08582798621876 402.7876862341025; 122.54568287977382 371.08582798621876 1156.545549635621 404.79439780397365; 115.31526599245463 402.7876862341025 404.79439780397365 829.8933758221423], [1631.311460636115 336.15706054385527 302.53764718806946 281.01944344319566; 336.1570605438553 2240.600708663522 1062.3874895759602 1129.0304343233577; 302.53764718806946 1062.3874895759602 2359.20908542582 452.0190754402239; 281.01944344319566 1129.0304343233577 452.0190754402239 1585.943164837384], [9262.734784434671 192.32792457293561 640.2147875336885 4281.532621735132; 192.32792457293561 29718.58058305707 2938.956083808823 34.91871865568121; 640.2147875336884 2938.956083808823 8550.450794983517 1294.7924247800922; 4281.532621735132 34.91871865568121 1294.7924247800922 4353.246261607954], [5275.579066366831 1404.022476678692 144.8544174690879 84.65866849531672; 1404.022476678692 2893.8569447678096 1350.0814380376473 1137.6907137204062; 144.8544174690879 1350.0814380376473 2426.02875785844 645.1324357344375; 84.65866849531672 1137.6907137204062 645.1324357344374 1338.7204582833865]], SubArray{Float64, 1, Matrix{Float64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[0.03631207419205951, 0.050532137525475626, -0.05919690085704499, -0.02345570553259435], [0.022381483177531747, 0.05584343156612803, 0.012304994058437657, -0.03035829188345396], [-0.004033528899298977, -0.05562870078293125, -0.031594664668368466, 0.026117608307547613], [-0.013937882491945351, -0.017242489070684375, -0.013986217544460278, 0.004197377910870159], [-0.0017265747252316888, -0.013667339271916226, 0.00518976419300432, 0.015935291833042098], [0.00711914061187624, 0.01669532291499687, -0.011596001652783627, 0.008639245899799697], [0.00369567401511485, -0.0014594707768407211, -0.0071396009484230305, 0.009977322338360572], [0.019906146912113343, 0.0019155468612339632, -0.05082548297901113, -0.021610456307453063], [0.013599691528992874, -0.002958827927533658, -0.05496295022887665, -0.02151460623813506], [0.008257653265075826, 0.011207412338922307, -0.021224594148147702, 0.0175430712654574]  …  [-0.01065191181482672, -0.014391173681816749, -0.018888173899533456, -0.0024329966732121733], [-0.0094370113208333, -0.06217420815871755, -0.04465346246731983, -0.0037311328234324037], [0.004139827799419127, -0.04022837800429224, -0.03801383375258638, -0.0008775074656099946], [-0.0062726909318362445, -0.04447897952980595, -0.007127897734317212, 0.006276372401689656], [-0.018993632447294795, -0.04825147141959052, -0.029367722703220522, -12.671725877801748], [-0.0146557879589031, -0.05414866783823391, -0.03838786873811273, -15.83602821929777], [-0.005765388082796363, -0.04822872254642441, -0.015804994188296284, 0.004818167149197805], [0.0018797724409228486, -0.013467394009664346, 0.004969611107007665, 0.010171768617283306], [-16.52354541323669, -0.05289433670461863, 0.06854793421832994, -0.04397166759326865], [-0.16577538547253035, -0.11545962378720276, -107.37909057514909, -65.5267184035896]], [0.999984077247513 0.9742443230122627 … 0.0 0.0; 0.9934251433547399 0.9864386931523093 … 0.0 0.0; … ; 0.0 0.0 … 0.0 0.0; 0.0 0.0 … 0.0 0.0;;;], SubArray{Float64, 2, Array{Float64, 3}, Tuple{Base.Slice{Base.OneTo{Int64}}, Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[0.999984077247513 0.9742443230122627 … 0.0 0.0; 0.9934251433547399 0.9864386931523093 … 0.0 0.0; … ; 0.0 0.0 … 0.0 0.0; 0.0 0.0 … 0.0 0.0]], [0.00021264759968354918 0.0001366312192898067 6.893326536645816e-5; -7.116216528202379e-5 -9.624603074709359e-5 -8.168702910338332e-5; … ; 0.0 0.0 0.0; 0.0 0.0 0.0], [0.00012193189761751377 9.136569716483578e-5 1.8942131309443444e-5; -9.172536249002494e-5 -3.223945158764563e-5 -1.6460619036706703e-5; … ; 0.0 0.0 0.0; 0.0 0.0 0.0], [-0.5 -0.5 -0.5 -0.5; -0.5 0.8333333333333334 -0.16666666666666666 -0.16666666666666666; -0.5 -0.16666666666666666 0.8333333333333334 -0.16666666666666666; -0.5 -0.16666666666666666 -0.16666666666666666 0.8333333333333334], [0.0; 0.0; 0.0; 0.0;;], [5275.579066366831; 2893.8569447678096; … ; 1338.7204582833865; 0.0;;], [-0.00012052768567257065; 0.0005243291188868516; … ; 1.0; 0.07759491792130875;;], [114151.93577995931; 112764.8322387724; … ; 0.0; 0.0;;], [0.5522037149772232; 0.5265175267226798; … ; -0.00868396722208467; -0.008645197003287726;;], [-0.00021765700313519876 0.00010560139303391282 … -0.1017151721365145 0.0; 0.00010560139303391282 -0.0003967945725390119 … 0.4424894271067903 0.0; … ; -0.1017151721365145 0.4424894271067903 … 843.9154171833779 65.48354752887124; 0.0 0.0 … 65.48354752887124 -0.0030486101809490407;;;], [7029.161398740817 3759.8136305049893 … -489.62335872731694 -489.3949460151179; 3759.8136305049893 5717.770656784291 … -527.4797583425097 -527.2632190631304; … ; -489.62335872731694 -527.4797583425097 … 108102.02109867314 108102.00248300146; -489.3949460151179 -527.2632190631304 … 108102.00248300146 -0.01612153787936449;;;], Bool[1; 1; 0; 0;;], SubArray{Float64, 1, Matrix{Float64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[0.0, 0.0, 0.0, 0.0]], SubArray{Float64, 1, Matrix{Float64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[5275.579066366831, 2893.8569447678096, 2426.02875785844, 1338.7204582833865, 0.0]], SubArray{Float64, 1, Matrix{Float64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[-0.00012052768567257065, 0.0005243291188868516, 0.0740234914604904, 1.0, 0.07759491792130875]], SubArray{Float64, 1, Matrix{Float64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[114151.93577995931, 112764.8322387724, 112402.00967340921, 108102.02109867314, 0.0, 0.0]], SubArray{Float64, 1, Matrix{Float64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[0.5522037149772232, 0.5265175267226798, 1.0, 0.00022851969219851807, -0.00868396722208467, -0.008645197003287726]], SubArray{Float64, 2, Array{Float64, 3}, Tuple{Base.Slice{Base.OneTo{Int64}}, Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[-0.00021765700313519876 0.00010560139303391282 … -0.1017151721365145 0.0; 0.00010560139303391282 -0.0003967945725390119 … 0.4424894271067903 0.0; … ; -0.1017151721365145 0.4424894271067903 … 843.9154171833779 65.48354752887124; 0.0 0.0 … 65.48354752887124 -0.0030486101809490407]], SubArray{Float64, 2, Array{Float64, 3}, Tuple{Base.Slice{Base.OneTo{Int64}}, Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[7029.161398740817 3759.8136305049893 … -489.62335872731694 -489.3949460151179; 3759.8136305049893 5717.770656784291 … -527.4797583425097 -527.2632190631304; … ; -489.62335872731694 -527.4797583425097 … 108102.02109867314 108102.00248300146; -489.3949460151179 -527.2632190631304 … 108102.00248300146 -0.01612153787936449]], SubArray{Bool, 1, Matrix{Bool}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[1, 1, 0, 0]], [4; 1; 2; 3;;], SubArray{Int64, 1, Matrix{Int64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[4, 1, 2, 3]], -1.5552194868097581e7, -1.5552193929721253e7), nothing, nothing)



The first argument is the path to the PLINK 1 `.bed`, and the second argument is the number of populations. The following keyword arguments are supported: 

- `T`: Precision of the estimation. `Float64` or `Float32`. Default `Float64`. 
- `use_gpu`: Whether to use GPU for estimation. Default `false`. 
- `rng`: Random number generator. Default `Random.GLOBAL_RNG`. 
- `prefix`: Prefix of the PLINK file only with the SNPs selected using SKFR. The output file is named `$(prefix)_$(K)_$(sparsity)aims.bed`. 
- `sparsity`: Number of SNPs selected by SKFR. Default `nothing` and skip SKFR. 
- `skfr_tries`: Runs SKFR this many times and choose the best clustering. Default 1. 
- `skfr_max_inner_iter`: Runs each SKFR for up to this many iterations or until convergence. Default 50. 
- `admix_n_iter`: Maximum number of iterations for ADMIXTURE. Default 1000. 
- `admix_rtol`: Convergence criteria in terms of relative change in loglikelihood. Default 1e-7. 
- `admix_n_em_iter`: Number of EM iterations to get a good initial guess for estimation. Default 5. 
- `Q`: Number of steps to be used in quasi-Newton acceleration. Default 3. 

The output are:
- `d`: the strucutre to store Admixture data. In particular, `d.p` stores the allele frequencies and `d.q` stores the admixture proportions. 
- `clusters`: cluster labels of each samples, `nothing` if `sparsity == nothing`.
- `aims`: The index of the selected SNPs in the decreasing order of importance, `nothing` if `sparsity == nothing`. 

To see the allele frequencies of the first allele listed in the `.bim` file accompanying the `.bed` file:


```julia
d.p
```




    4×54051 Matrix{Float64}:
     0.841704  0.971913  0.991518  0.546057  …  0.920156  0.99999   0.990385
     0.923591  0.992092  0.939205  0.471689     0.942705  0.993918  0.92073
     0.917793  0.982938  0.959373  0.565633     0.947937  0.981754  0.99999
     0.912906  0.999763  0.935531  0.456692     0.968599  0.969226  0.99999



To see the admixture proportion of each sample:


```julia
d.q
```




    4×379 Matrix{Float64}:
     0.0709719  0.0344815  0.0801809  0.116321  …  0.098118   1.0e-5   1.0e-5
     0.417541   0.74624    0.714427   0.368537     0.0793005  1.0e-5   1.0e-5
     1.0e-5     0.219269   0.205382   1.0e-5       0.822571   1.0e-5   1.0e-5
     0.511478   1.0e-5     1.0e-5     0.515132     1.0e-5     0.99997  0.99997



The following shows the final loglikelihood of the parameters.


```julia
d.ll_new
```




    -1.5552193929721253e7



The following is an example with `sparsity` defined. 


```julia
d, clusters, aims = OpenADMIXTURE.run_admixture(filename, 4; T=Float64, use_gpu=false, rng=MersenneTwister(7856), sparsity=10000, admix_rtol=1e-7, prefix="./EUR_subset")
```

    cnt of sparse1:6
    ./EUR_subset_4_10000aims
      0.016605 seconds (4 allocations: 64 bytes)
      0.017555 seconds (4 allocations: 64 bytes)
      0.016559 seconds (4 allocations: 64 bytes)
      0.017248 seconds (4 allocations: 64 bytes)
      0.016535 seconds (4 allocations: 64 bytes)
      0.017239 seconds (4 allocations: 64 bytes)
      0.016542 seconds (4 allocations: 64 bytes)
      0.017329 seconds (4 allocations: 64 bytes)
      0.016328 seconds (4 allocations: 64 bytes)
      0.017016 seconds (4 allocations: 64 bytes)
      0.207011 seconds (434 allocations: 19.938 KiB)
    initial ll: -3.077259034240569e6
    -3.077259034240569e6
      0.051540 seconds (4 allocations: 64 bytes)
      0.000510 seconds (2.66 k allocations: 125.109 KiB)
      0.068163 seconds (4 allocations: 64 bytes)
      0.006425 seconds (8 allocations: 768 bytes)
    -3.077259034240569e6
      0.051073 seconds (4 allocations: 64 bytes)
      0.000492 seconds (2.66 k allocations: 125.109 KiB)
      0.068530 seconds (4 allocations: 64 bytes)
      0.006167 seconds (8 allocations: 768 bytes)
    -3.077259034240569e6
    -3.0280405173204164e6
    -3.0300059018500494e6
    Iteration 1: ll = -3.0300059018500494e6, reldiff = 0.015355591409346895
      0.331178 seconds (8.49 k allocations: 1.342 MiB)
    
    
    -3.0300059018500494e6
      0.056997 seconds (4 allocations: 64 bytes)
      0.000501 seconds (2.66 k allocations: 125.109 KiB)
      0.070048 seconds (4 allocations: 64 bytes)
      0.005810 seconds (8 allocations: 768 bytes)
    -3.0300059018500494e6
      0.052859 seconds (4 allocations: 64 bytes)
      0.000508 seconds (2.66 k allocations: 125.109 KiB)
      0.068050 seconds (4 allocations: 64 bytes)
      0.005522 seconds (8 allocations: 768 bytes)
    -3.0300059018500494e6
    -3.0057198788971743e6
    -3.003246863866929e6
    Iteration 2: ll = -3.003246863866929e6, reldiff = 0.008831348469249512
      0.335381 seconds (8.49 k allocations: 1.658 MiB)
    
    
    -3.003246863866929e6
      0.051635 seconds (4 allocations: 64 bytes)
      0.000634 seconds (2.66 k allocations: 125.109 KiB)
      0.068657 seconds (4 allocations: 64 bytes)
      0.005354 seconds (8 allocations: 768 bytes)
    -3.003246863866929e6
      0.051419 seconds (4 allocations: 64 bytes)
      0.000527 seconds (2.66 k allocations: 125.109 KiB)
      0.068346 seconds (4 allocations: 64 bytes)
      0.005266 seconds (8 allocations: 768 bytes)
    -3.003246863866929e6
    -2.9954683999207392e6
    -2.9951395344067365e6
    Iteration 3: ll = -2.9951395344067365e6, reldiff = 0.0026995214937987526
      0.326860 seconds (8.49 k allocations: 1.977 MiB)
    
    
    -2.9951395344067365e6
      0.054100 seconds (4 allocations: 64 bytes)
      0.001317 seconds (2.66 k allocations: 125.109 KiB)
      0.068212 seconds (4 allocations: 64 bytes)
      0.005262 seconds (8 allocations: 768 bytes)
    -2.9951395344067365e6
      0.052234 seconds (4 allocations: 64 bytes)
      0.001201 seconds (2.66 k allocations: 125.109 KiB)
      0.068150 seconds (4 allocations: 64 bytes)
      0.005203 seconds (8 allocations: 768 bytes)
    -2.9951395344067365e6
    -2.992523970635955e6
    -2.993074174142398e6
    Iteration 4: ll = -2.993074174142398e6, reldiff = 0.0006895706328912819
      0.331379 seconds (8.49 k allocations: 1.975 MiB)
    
    
    -2.993074174142398e6
      0.053118 seconds (4 allocations: 64 bytes)
      0.001216 seconds (2.66 k allocations: 125.109 KiB)
      0.067760 seconds (4 allocations: 64 bytes)
      0.005182 seconds (8 allocations: 768 bytes)
    -2.993074174142398e6
      0.051179 seconds (4 allocations: 64 bytes)
      0.001521 seconds (2.66 k allocations: 125.109 KiB)
      0.068610 seconds (4 allocations: 64 bytes)
      0.005191 seconds (8 allocations: 768 bytes)
    -2.993074174142398e6
    -2.9914347372060185e6
    -2.9912617609358244e6
    Iteration 5: ll = -2.9912617609358244e6, reldiff = 0.0006055356804155662
      0.329000 seconds (8.49 k allocations: 1.975 MiB)
    
    
    -2.9912617609358244e6
      0.051459 seconds (4 allocations: 64 bytes)
      0.001472 seconds (2.66 k allocations: 125.109 KiB)
      0.068897 seconds (4 allocations: 64 bytes)
      0.005212 seconds (8 allocations: 768 bytes)
    -2.9912617609358244e6
      0.051166 seconds (4 allocations: 64 bytes)
      0.001426 seconds (2.66 k allocations: 125.109 KiB)
      0.068332 seconds (4 allocations: 64 bytes)
      0.005103 seconds (8 allocations: 768 bytes)
    -2.9912617609358244e6
    -2.9907587680529426e6
    -2.9919207918732297e6
    Iteration 6: ll = -2.9907587680529426e6, reldiff = 0.00016815408449055843
      0.328581 seconds (8.49 k allocations: 1.975 MiB)
    
    
    -2.9907587680529426e6
      0.051610 seconds (4 allocations: 64 bytes)
      0.001440 seconds (2.66 k allocations: 125.109 KiB)
      0.068850 seconds (4 allocations: 64 bytes)
      0.005143 seconds (8 allocations: 768 bytes)
    -2.9907587680529426e6
      0.051673 seconds (4 allocations: 64 bytes)
      0.001431 seconds (2.66 k allocations: 125.109 KiB)
      0.068392 seconds (4 allocations: 64 bytes)
      0.006488 seconds (8 allocations: 768 bytes)
    -2.9907587680529426e6
    -2.9904190503580016e6
    -2.992138931188849e6
    Iteration 7: ll = -2.9904190503580016e6, reldiff = 0.000113589132821348
      0.330491 seconds (8.49 k allocations: 1.984 MiB)
    
    
    -2.9904190503580016e6
      0.051826 seconds (4 allocations: 64 bytes)
      0.001549 seconds (2.66 k allocations: 125.109 KiB)
      0.068626 seconds (4 allocations: 64 bytes)
      0.005074 seconds (8 allocations: 768 bytes)
    -2.9904190503580016e6
      0.051142 seconds (4 allocations: 64 bytes)
      0.001537 seconds (2.66 k allocations: 125.109 KiB)
      0.068163 seconds (4 allocations: 64 bytes)
      0.005082 seconds (8 allocations: 768 bytes)
    -2.9904190503580016e6
    -2.9901733383521987e6
    -2.99042389440459e6
    Iteration 8: ll = -2.9901733383521987e6, reldiff = 8.216641268837562e-5
      0.328440 seconds (8.49 k allocations: 1.975 MiB)
    
    
    -2.9901733383521987e6
      0.051310 seconds (4 allocations: 64 bytes)
      0.001440 seconds (2.66 k allocations: 125.109 KiB)
      0.068794 seconds (4 allocations: 64 bytes)
      0.005056 seconds (8 allocations: 768 bytes)
    -2.9901733383521987e6
      0.051610 seconds (4 allocations: 64 bytes)
      0.001434 seconds (2.66 k allocations: 125.109 KiB)
      0.068217 seconds (4 allocations: 64 bytes)
      0.005080 seconds (8 allocations: 768 bytes)
    -2.9901733383521987e6
    -2.990000767067695e6
    -2.990061270031604e6
    Iteration 9: ll = -2.990061270031604e6, reldiff = 3.747887092608675e-5
      0.328438 seconds (8.49 k allocations: 1.975 MiB)
    
    
    -2.990061270031604e6
      0.051703 seconds (4 allocations: 64 bytes)
      0.001236 seconds (2.66 k allocations: 125.109 KiB)
      0.068567 seconds (4 allocations: 64 bytes)
      0.005093 seconds (8 allocations: 768 bytes)
    -2.990061270031604e6
      0.051414 seconds (4 allocations: 64 bytes)
      0.001220 seconds (2.66 k allocations: 125.109 KiB)
      0.068460 seconds (4 allocations: 64 bytes)
      0.005338 seconds (8 allocations: 768 bytes)
    -2.990061270031604e6
    -2.98974042373545e6
    -2.989726746092522e6
    Iteration 10: ll = -2.989726746092522e6, reldiff = 0.00011187862350338979
      0.336451 seconds (8.49 k allocations: 1.975 MiB)
    
    
    -2.989726746092522e6
      0.051309 seconds (4 allocations: 64 bytes)
      0.001235 seconds (2.66 k allocations: 125.109 KiB)
      0.068019 seconds (4 allocations: 64 bytes)
      0.005074 seconds (8 allocations: 768 bytes)
    -2.989726746092522e6
      0.051070 seconds (4 allocations: 64 bytes)
      0.001263 seconds (2.66 k allocations: 125.109 KiB)
      0.067901 seconds (4 allocations: 64 bytes)
      0.005053 seconds (8 allocations: 768 bytes)
    -2.989726746092522e6
    -2.989633149826683e6
    -2.989850885712909e6
    Iteration 11: ll = -2.989633149826683e6, reldiff = 3.1305959971517296e-5
      0.326089 seconds (8.49 k allocations: 1.975 MiB)
    
    
    -2.989633149826683e6
      0.051355 seconds (4 allocations: 64 bytes)
      0.001224 seconds (2.66 k allocations: 125.109 KiB)
      0.069572 seconds (4 allocations: 64 bytes)
      0.005054 seconds (8 allocations: 768 bytes)
    -2.989633149826683e6
      0.051855 seconds (4 allocations: 64 bytes)
      0.001561 seconds (2.66 k allocations: 125.109 KiB)
      0.068535 seconds (4 allocations: 64 bytes)
      0.005109 seconds (8 allocations: 768 bytes)
    -2.989633149826683e6
    -2.9895800598967243e6
    -2.9910787931252318e6
    Iteration 12: ll = -2.9895800598967243e6, reldiff = 1.7758008189694848e-5
      0.329700 seconds (8.49 k allocations: 1.975 MiB)
    
    
    -2.9895800598967243e6
      0.053446 seconds (4 allocations: 64 bytes)
      0.001343 seconds (2.66 k allocations: 125.109 KiB)
      0.068643 seconds (4 allocations: 64 bytes)
      0.005082 seconds (8 allocations: 768 bytes)
    -2.9895800598967243e6
      0.051077 seconds (4 allocations: 64 bytes)
      0.001326 seconds (2.66 k allocations: 125.109 KiB)
      0.068270 seconds (4 allocations: 64 bytes)
      0.005109 seconds (8 allocations: 768 bytes)
    -2.9895800598967243e6
    -2.9895361151002464e6
    -3.0470678619324937e6
    Iteration 13: ll = -2.9895361151002464e6, reldiff = 1.4699320840207977e-5
      0.330408 seconds (8.49 k allocations: 1.975 MiB)
    
    
    -2.9895361151002464e6
      0.051299 seconds (4 allocations: 64 bytes)
      0.001231 seconds (2.66 k allocations: 125.109 KiB)
      0.068383 seconds (4 allocations: 64 bytes)
      0.005068 seconds (8 allocations: 768 bytes)
    -2.9895361151002464e6
      0.051155 seconds (4 allocations: 64 bytes)
      0.001107 seconds (2.66 k allocations: 125.109 KiB)
      0.068226 seconds (4 allocations: 64 bytes)
      0.005089 seconds (8 allocations: 768 bytes)
    -2.9895361151002464e6
    -2.989495350413259e6
    -4.870537010936856e6
    Iteration 14: ll = -2.989495350413259e6, reldiff = 1.363579010849405e-5
      0.326845 seconds (8.49 k allocations: 1.975 MiB)
    
    
    -2.989495350413259e6
      0.051376 seconds (4 allocations: 64 bytes)
      0.001114 seconds (2.66 k allocations: 125.109 KiB)
      0.068882 seconds (4 allocations: 64 bytes)
      0.005072 seconds (8 allocations: 768 bytes)
    -2.989495350413259e6
      0.051083 seconds (4 allocations: 64 bytes)
      0.001111 seconds (2.66 k allocations: 125.109 KiB)
      0.068961 seconds (4 allocations: 64 bytes)
      0.005080 seconds (8 allocations: 768 bytes)
    -2.989495350413259e6
    -2.989455754492594e6
    -3.204234679307862e6
    Iteration 15: ll = -2.989455754492594e6, reldiff = 1.3245018313781956e-5
      0.328163 seconds (8.49 k allocations: 1.975 MiB)
    
    
    -2.989455754492594e6
      0.051956 seconds (4 allocations: 64 bytes)
      0.001235 seconds (2.66 k allocations: 125.109 KiB)
      0.068537 seconds (4 allocations: 64 bytes)
      0.005054 seconds (8 allocations: 768 bytes)
    -2.989455754492594e6
      0.051440 seconds (4 allocations: 64 bytes)
      0.001220 seconds (2.66 k allocations: 125.109 KiB)
      0.068383 seconds (4 allocations: 64 bytes)
      0.005095 seconds (8 allocations: 768 bytes)
    -2.989455754492594e6
    -2.9894188142380556e6
    -3.005116706744177e6
    Iteration 16: ll = -2.9894188142380556e6, reldiff = 1.2356849397354584e-5
      0.329269 seconds (8.60 k allocations: 2.031 MiB)
    
    
    -2.9894188142380556e6
      0.060329 seconds (4 allocations: 64 bytes)
      0.001310 seconds (2.66 k allocations: 125.109 KiB)
      0.070944 seconds (4 allocations: 64 bytes)
      0.005112 seconds (8 allocations: 768 bytes)
    -2.9894188142380556e6
      0.051192 seconds (4 allocations: 64 bytes)
      0.001234 seconds (2.66 k allocations: 125.109 KiB)
      0.069936 seconds (4 allocations: 64 bytes)
      0.005086 seconds (8 allocations: 768 bytes)
    -2.9894188142380556e6
    -2.9893843473597807e6
    -3.0024701291123424e6
    Iteration 17: ll = -2.9893843473597807e6, reldiff = 1.152962512671757e-5
      0.341146 seconds (8.49 k allocations: 1.976 MiB)
    
    
    -2.9893843473597807e6
      0.051523 seconds (4 allocations: 64 bytes)
      0.001348 seconds (2.66 k allocations: 125.109 KiB)
      0.068716 seconds (4 allocations: 64 bytes)
      0.005079 seconds (8 allocations: 768 bytes)
    -2.9893843473597807e6
      0.051583 seconds (4 allocations: 64 bytes)
      0.001335 seconds (2.66 k allocations: 125.109 KiB)
      0.067773 seconds (4 allocations: 64 bytes)
      0.005060 seconds (8 allocations: 768 bytes)
    -2.9893843473597807e6
    -2.9893519062841535e6
    -3.2076681927966746e6
    Iteration 18: ll = -2.9893519062841535e6, reldiff = 1.0852092557411792e-5
      0.327834 seconds (8.49 k allocations: 1.977 MiB)
    
    
    -2.9893519062841535e6
      0.051513 seconds (4 allocations: 64 bytes)
      0.001347 seconds (2.66 k allocations: 125.109 KiB)
      0.068581 seconds (4 allocations: 64 bytes)
      0.005065 seconds (8 allocations: 768 bytes)
    -2.9893519062841535e6
      0.051358 seconds (4 allocations: 64 bytes)
      0.001324 seconds (2.66 k allocations: 125.109 KiB)
      0.067730 seconds (4 allocations: 64 bytes)
      0.005019 seconds (8 allocations: 768 bytes)
    -2.9893519062841535e6
    -2.989320160419182e6
    -4.216603856297227e6
    Iteration 19: ll = -2.989320160419182e6, reldiff = 1.0619647992866069e-5
      0.327422 seconds (8.49 k allocations: 1.979 MiB)
    
    
    -2.989320160419182e6
      0.053565 seconds (4 allocations: 64 bytes)
      0.001343 seconds (2.66 k allocations: 125.109 KiB)
      0.068597 seconds (4 allocations: 64 bytes)
      0.005200 seconds (8 allocations: 768 bytes)
    -2.989320160419182e6
      0.051766 seconds (4 allocations: 64 bytes)
      0.001229 seconds (2.66 k allocations: 125.109 KiB)
      0.068190 seconds (4 allocations: 64 bytes)
      0.005048 seconds (8 allocations: 768 bytes)
    -2.989320160419182e6
    -2.9892881573209567e6
    -3.0190224119707076e6
    Iteration 20: ll = -2.9892881573209567e6, reldiff = 1.0705811524963374e-5
      0.330434 seconds (8.49 k allocations: 1.975 MiB)
    
    
    -2.9892881573209567e6
      0.051287 seconds (4 allocations: 64 bytes)
      0.001235 seconds (2.66 k allocations: 125.109 KiB)
      0.068540 seconds (4 allocations: 64 bytes)
      0.005040 seconds (8 allocations: 768 bytes)
    -2.9892881573209567e6
      0.051115 seconds (4 allocations: 64 bytes)
      0.001226 seconds (2.66 k allocations: 125.109 KiB)
      0.068497 seconds (4 allocations: 64 bytes)
      0.005067 seconds (8 allocations: 768 bytes)
    -2.9892881573209567e6
    -2.989255848399305e6
    -3.070369539860443e6
    Iteration 21: ll = -2.989255848399305e6, reldiff = 1.0808232579648808e-5
      0.327615 seconds (8.49 k allocations: 1.975 MiB)
    
    
    -2.989255848399305e6
      0.055029 seconds (4 allocations: 64 bytes)
      0.001283 seconds (2.66 k allocations: 125.109 KiB)
      0.068664 seconds (4 allocations: 64 bytes)
      0.005064 seconds (8 allocations: 768 bytes)
    -2.989255848399305e6
      0.051607 seconds (4 allocations: 64 bytes)
      0.001233 seconds (2.66 k allocations: 125.109 KiB)
      0.068599 seconds (4 allocations: 64 bytes)
      0.005057 seconds (8 allocations: 768 bytes)
    -2.989255848399305e6
    -2.989223200480792e6
    -2.721573989958778e7
    Iteration 22: ll = -2.989223200480792e6, reldiff = 1.0921754499555953e-5
      0.332062 seconds (8.49 k allocations: 1.975 MiB)
    
    
    -2.989223200480792e6
      0.051913 seconds (4 allocations: 64 bytes)
      0.001239 seconds (2.66 k allocations: 125.109 KiB)
      0.068575 seconds (4 allocations: 64 bytes)
      0.005057 seconds (8 allocations: 768 bytes)
    -2.989223200480792e6
      0.051357 seconds (4 allocations: 64 bytes)
      0.001113 seconds (2.66 k allocations: 125.109 KiB)
      0.067779 seconds (4 allocations: 64 bytes)
      0.004986 seconds (8 allocations: 768 bytes)
    -2.989223200480792e6
    -2.98918862571243e6
    -3.0027735217421465e6
    Iteration 23: ll = -2.98918862571243e6, reldiff = 1.1566472639627613e-5
      0.327517 seconds (8.49 k allocations: 1.983 MiB)
    
    
    -2.98918862571243e6
      0.051341 seconds (4 allocations: 64 bytes)
      0.001012 seconds (2.66 k allocations: 125.109 KiB)
      0.068575 seconds (4 allocations: 64 bytes)
      0.005209 seconds (8 allocations: 768 bytes)
    -2.98918862571243e6
      0.051170 seconds (4 allocations: 64 bytes)
      0.001009 seconds (2.66 k allocations: 125.109 KiB)
      0.067795 seconds (4 allocations: 64 bytes)
      0.005078 seconds (8 allocations: 768 bytes)
    -2.98918862571243e6
    -2.9891504427890005e6
    -3.055894439483546e6
    Iteration 24: ll = -2.9891504427890005e6, reldiff = 1.2773674802989472e-5
      0.326734 seconds (8.49 k allocations: 1.975 MiB)
    
    
    -2.9891504427890005e6
      0.052237 seconds (4 allocations: 64 bytes)
      0.001021 seconds (2.66 k allocations: 125.109 KiB)
      0.068584 seconds (4 allocations: 64 bytes)
      0.005008 seconds (8 allocations: 768 bytes)
    -2.9891504427890005e6
      0.051356 seconds (4 allocations: 64 bytes)
      0.001006 seconds (2.66 k allocations: 125.109 KiB)
      0.067855 seconds (4 allocations: 64 bytes)
      0.005038 seconds (8 allocations: 768 bytes)
    -2.9891504427890005e6
    -2.9891102235109448e6
    -2.9937728315902813e6
    Iteration 25: ll = -2.9891102235109448e6, reldiff = 1.3455086595836446e-5
      0.327716 seconds (8.49 k allocations: 1.975 MiB)
    
    
    -2.9891102235109448e6
      0.051503 seconds (4 allocations: 64 bytes)
      0.000797 seconds (2.66 k allocations: 125.109 KiB)
      0.072497 seconds (4 allocations: 64 bytes)
      0.005026 seconds (8 allocations: 768 bytes)
    -2.9891102235109448e6
      0.051099 seconds (4 allocations: 64 bytes)
      0.000790 seconds (2.66 k allocations: 125.109 KiB)
      0.067850 seconds (4 allocations: 64 bytes)
      0.005050 seconds (8 allocations: 768 bytes)
    -2.9891102235109448e6
    -2.989069852298082e6
    -2.993139156894906e6
    Iteration 26: ll = -2.989069852298082e6, reldiff = 1.3506097080447741e-5
      0.330044 seconds (8.49 k allocations: 1.975 MiB)
    
    
    -2.989069852298082e6
      0.051596 seconds (4 allocations: 64 bytes)
      0.000912 seconds (2.66 k allocations: 125.109 KiB)
      0.068444 seconds (4 allocations: 64 bytes)
      0.005081 seconds (8 allocations: 768 bytes)
    -2.989069852298082e6
      0.051489 seconds (4 allocations: 64 bytes)
      0.000898 seconds (2.66 k allocations: 125.109 KiB)
      0.068667 seconds (4 allocations: 64 bytes)
      0.005089 seconds (8 allocations: 768 bytes)
    -2.989069852298082e6
    -2.9890318651039726e6
    -2.991710920354535e6
    Iteration 27: ll = -2.9890318651039726e6, reldiff = 1.2708700694979335e-5
      0.327652 seconds (8.49 k allocations: 1.975 MiB)
    
    
    -2.9890318651039726e6
      0.051948 seconds (4 allocations: 64 bytes)
      0.000903 seconds (2.66 k allocations: 125.109 KiB)
      0.068620 seconds (4 allocations: 64 bytes)
      0.005098 seconds (8 allocations: 768 bytes)
    -2.9890318651039726e6
      0.051546 seconds (4 allocations: 64 bytes)
      0.000907 seconds (2.66 k allocations: 125.109 KiB)
      0.068082 seconds (4 allocations: 64 bytes)
      0.005061 seconds (8 allocations: 768 bytes)
    -2.9890318651039726e6
    -2.9889978041921854e6
    -2.9892098593712384e6
    Iteration 28: ll = -2.9889978041921854e6, reldiff = 1.1395298987895992e-5
      0.327987 seconds (8.49 k allocations: 1.975 MiB)
    
    
    -2.9889978041921854e6
      0.051315 seconds (4 allocations: 64 bytes)
      0.001014 seconds (2.66 k allocations: 125.109 KiB)
      0.068676 seconds (4 allocations: 64 bytes)
      0.005056 seconds (8 allocations: 768 bytes)
    -2.9889978041921854e6
      0.051113 seconds (4 allocations: 64 bytes)
      0.000999 seconds (2.66 k allocations: 125.109 KiB)
      0.068374 seconds (4 allocations: 64 bytes)
      0.005035 seconds (8 allocations: 768 bytes)
    -2.9889978041921854e6
    -2.9889710379184415e6
    -2.9892181158235935e6
    Iteration 29: ll = -2.9889710379184415e6, reldiff = 8.954932555118614e-6
      0.327091 seconds (8.49 k allocations: 1.975 MiB)
    
    
    -2.9889710379184415e6
      0.051287 seconds (4 allocations: 64 bytes)
      0.001010 seconds (2.66 k allocations: 125.109 KiB)
      0.067741 seconds (4 allocations: 64 bytes)
      0.005077 seconds (8 allocations: 768 bytes)
    -2.9889710379184415e6
      0.051320 seconds (4 allocations: 64 bytes)
      0.001005 seconds (2.66 k allocations: 125.109 KiB)
      0.068327 seconds (4 allocations: 64 bytes)
      0.005077 seconds (8 allocations: 768 bytes)
    -2.9889710379184415e6
    -2.9889486095322985e6
    -2.9900355066040237e6
    Iteration 30: ll = -2.9889486095322985e6, reldiff = 7.503714776226964e-6
      0.326470 seconds (8.49 k allocations: 1.975 MiB)
    
    
    -2.9889486095322985e6
      0.051307 seconds (4 allocations: 64 bytes)
      0.001011 seconds (2.66 k allocations: 125.109 KiB)
      0.068545 seconds (4 allocations: 64 bytes)
      0.005031 seconds (8 allocations: 768 bytes)
    -2.9889486095322985e6
      0.051066 seconds (4 allocations: 64 bytes)
      0.000996 seconds (2.66 k allocations: 125.109 KiB)
      0.068230 seconds (4 allocations: 64 bytes)
      0.005030 seconds (8 allocations: 768 bytes)
    -2.9889486095322985e6
    -2.988929193837599e6
    -2.9900550381234307e6
    Iteration 31: ll = -2.988929193837599e6, reldiff = 6.495827542066974e-6
      0.329288 seconds (8.49 k allocations: 1.975 MiB)
    
    
    -2.988929193837599e6
      0.051494 seconds (4 allocations: 64 bytes)
      0.001013 seconds (2.66 k allocations: 125.109 KiB)
      0.068373 seconds (4 allocations: 64 bytes)
      0.005045 seconds (8 allocations: 768 bytes)
    -2.988929193837599e6
      0.051083 seconds (4 allocations: 64 bytes)
      0.001001 seconds (2.66 k allocations: 125.109 KiB)
      0.067739 seconds (4 allocations: 64 bytes)
      0.005013 seconds (8 allocations: 768 bytes)
    -2.988929193837599e6
    -2.988913407219005e6
    -2.9889200662938342e6
    Iteration 32: ll = -2.9889200662938342e6, reldiff = 3.053783871318594e-6
      0.326375 seconds (8.49 k allocations: 1.975 MiB)
    
    
    -2.9889200662938342e6
      0.051381 seconds (4 allocations: 64 bytes)
      0.000797 seconds (2.66 k allocations: 125.109 KiB)
      0.070106 seconds (4 allocations: 64 bytes)
      0.005042 seconds (8 allocations: 768 bytes)
    -2.9889200662938342e6
      0.051473 seconds (4 allocations: 64 bytes)
      0.001423 seconds (2.66 k allocations: 125.109 KiB)
      0.069658 seconds (4 allocations: 64 bytes)
      0.005092 seconds (8 allocations: 768 bytes)
    -2.9889200662938342e6
    -2.9888740258513074e6
    -2.9888789198555816e6
    Iteration 33: ll = -2.9888789198555816e6, reldiff = 1.3766322731964776e-5
      0.332496 seconds (8.60 k allocations: 2.032 MiB)
    
    
    -2.9888789198555816e6
      0.051510 seconds (4 allocations: 64 bytes)
      0.000801 seconds (2.66 k allocations: 125.109 KiB)
      0.068655 seconds (4 allocations: 64 bytes)
      0.005081 seconds (8 allocations: 768 bytes)
    -2.9888789198555816e6
      0.051496 seconds (4 allocations: 64 bytes)
      0.000782 seconds (2.66 k allocations: 125.109 KiB)
      0.068511 seconds (4 allocations: 64 bytes)
      0.005005 seconds (8 allocations: 768 bytes)
    -2.9888789198555816e6
    -2.9888553511574874e6
    -2.988873849177302e6
    Iteration 34: ll = -2.988873849177302e6, reldiff = 1.696515120117397e-6
      0.327016 seconds (8.49 k allocations: 1.976 MiB)
    
    
    -2.988873849177302e6
      0.051493 seconds (4 allocations: 64 bytes)
      0.000796 seconds (2.66 k allocations: 125.109 KiB)
      0.068231 seconds (4 allocations: 64 bytes)
      0.005037 seconds (8 allocations: 768 bytes)
    -2.988873849177302e6
      0.051156 seconds (4 allocations: 64 bytes)
      0.000784 seconds (2.66 k allocations: 125.109 KiB)
      0.068227 seconds (4 allocations: 64 bytes)
      0.004956 seconds (8 allocations: 768 bytes)
    -2.988873849177302e6
    -2.9888501509662215e6
    -2.9888497143354164e6
    Iteration 35: ll = -2.9888497143354164e6, reldiff = 8.074894794258681e-6
      0.326043 seconds (8.49 k allocations: 1.977 MiB)
    
    
    -2.9888497143354164e6
      0.051530 seconds (4 allocations: 64 bytes)
      0.000795 seconds (2.66 k allocations: 125.109 KiB)
      0.068550 seconds (4 allocations: 64 bytes)
      0.005018 seconds (8 allocations: 768 bytes)
    -2.9888497143354164e6
      0.051266 seconds (4 allocations: 64 bytes)
      0.000788 seconds (2.66 k allocations: 125.109 KiB)
      0.068305 seconds (4 allocations: 64 bytes)
      0.005013 seconds (8 allocations: 768 bytes)
    -2.9888497143354164e6
    -2.98884770192568e6
    -2.98884653275144e6
    Iteration 36: ll = -2.98884653275144e6, reldiff = 1.0644844272742283e-6
      0.328312 seconds (8.49 k allocations: 1.975 MiB)
    
    
    -2.98884653275144e6
      0.051698 seconds (4 allocations: 64 bytes)
      0.000808 seconds (2.66 k allocations: 125.109 KiB)
      0.068499 seconds (4 allocations: 64 bytes)
      0.005027 seconds (8 allocations: 768 bytes)
    -2.98884653275144e6
      0.051365 seconds (4 allocations: 64 bytes)
      0.000783 seconds (2.66 k allocations: 125.109 KiB)
      0.068251 seconds (4 allocations: 64 bytes)
      0.005036 seconds (8 allocations: 768 bytes)
    -2.98884653275144e6
    -2.9888461026714854e6
    -2.9888459670782355e6
    Iteration 37: ll = -2.9888459670782355e6, reldiff = 1.8926137505086115e-7
      0.326842 seconds (8.49 k allocations: 1.979 MiB)
    
    
    -2.9888459670782355e6
      0.051509 seconds (4 allocations: 64 bytes)
      0.000800 seconds (2.66 k allocations: 125.109 KiB)
      0.068580 seconds (4 allocations: 64 bytes)
      0.005017 seconds (8 allocations: 768 bytes)
    -2.9888459670782355e6
      0.051076 seconds (4 allocations: 64 bytes)
      0.000784 seconds (2.66 k allocations: 125.109 KiB)
      0.068157 seconds (4 allocations: 64 bytes)
      0.005003 seconds (8 allocations: 768 bytes)
    -2.9888459670782355e6
    -2.9888457179705338e6
    -2.9888456882726927e6
    Iteration 38: ll = -2.9888456882726927e6, reldiff = 9.328200442854993e-8
     12.511960 seconds (325.18 k allocations: 74.353 MiB)





    (AdmixData{Float64}(379, 10000, 4, 3, true, [0.582214916426889 1.0000000000027756e-5 … 0.9327568172111603 0.99999; 0.14092902448354105 0.41258807922799495 … 0.8596468613040379 0.8877110943870553; 0.05402188629156628 0.1339463980931601 … 0.9599270223437033 0.99999; 0.2228341727980035 0.45345552267884487 … 0.8107491631733151 0.99999], [0.5822686983061394 1.0e-5 … 0.932690502090477 0.99999; 0.14157978700364626 0.4122387868461372 … 0.8596295407288184 0.8876821133904721; 0.05422353338551516 0.13369030486485345 … 0.9604830519644512 0.99999; 0.2219279813046991 0.45406090828900936 … 0.8106984216220864 0.99999], [0.582226360690002 1.0e-5 … 0.9326919414077695 0.99999; 0.14088115931089432 0.4123807383809744 … 0.8596604973569697 0.8876695309831731; 0.05418392365429152 0.13379645401479928 … 0.9604303962397303 0.99999; 0.2227085563448121 0.45381280760422626 … 0.8106865432804832 0.99999], [0.582214916426889 1.0000000000027756e-5 … 0.9327568172111603 0.99999; 0.14092902448354105 0.41258807922799495 … 0.8596468613040379 0.8877110943870553; 0.05402188629156628 0.1339463980931601 … 0.9599270223437033 0.99999; 0.2228341727980035 0.45345552267884487 … 0.8107491631733151 0.99999], [0.582214916426889, 0.14092902448354105, 0.05402188629156628, 0.2228341727980035, 1.0000000000027756e-5, 0.41258807922799495, 0.1339463980931601, 0.45345552267884487, 1.0e-5, 0.5069557466869863  …  0.9785293750103126, 0.7676496020277787, 0.9327568172111603, 0.8596468613040379, 0.9599270223437033, 0.8107491631733151, 0.99999, 0.8877110943870553, 0.99999, 0.99999], [0.5822686983061394, 0.14157978700364626, 0.05422353338551516, 0.2219279813046991, 1.0e-5, 0.4122387868461372, 0.13369030486485345, 0.45406090828900936, 1.0e-5, 0.5061059458488054  …  0.9782466651408858, 0.7677475795250771, 0.932690502090477, 0.8596295407288184, 0.9604830519644512, 0.8106984216220864, 0.99999, 0.8876821133904721, 0.99999, 0.99999], [0.582226360690002, 0.14088115931089432, 0.05418392365429152, 0.2227085563448121, 1.0e-5, 0.4123807383809744, 0.13379645401479928, 0.45381280760422626, 1.0e-5, 0.5063020337105326  …  0.9784626120269361, 0.767673260692788, 0.9326919414077695, 0.8596604973569697, 0.9604303962397303, 0.8106865432804832, 0.99999, 0.8876695309831731, 0.99999, 0.99999], [0.582214916426889, 0.14092902448354105, 0.05402188629156628, 0.2228341727980035, 1.0000000000027756e-5, 0.41258807922799495, 0.1339463980931601, 0.45345552267884487, 1.0e-5, 0.5069557466869863  …  0.9785293750103126, 0.7676496020277787, 0.9327568172111603, 0.8596468613040379, 0.9599270223437033, 0.8107491631733151, 0.99999, 0.8877110943870553, 0.99999, 0.99999], [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0  …  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0  …  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0], [0.582214916426889 1.0000000000027756e-5 … 0.9999699999999999 0.9999699999999999; 0.14092902448354105 0.41258807922799495 … 1.0e-5 1.0e-5; 0.05402188629156628 0.1339463980931601 … 1.0e-5 1.0e-5; 0.2228341727980035 0.45345552267884487 … 1.0e-5 1.0e-5], [0.5822686983061394 1.0e-5 … 0.9999699999999999 0.9999699999999999; 0.14157978700364626 0.4122387868461372 … 1.0e-5 1.0e-5; 0.05422353338551516 0.13369030486485345 … 1.0e-5 1.0e-5; 0.2219279813046991 0.45406090828900936 … 1.0e-5 1.0e-5], [0.582226360690002 1.0e-5 … 0.9999699999999999 0.9999699999999999; 0.14088115931089432 0.4123807383809744 … 1.0e-5 1.0e-5; 0.05418392365429152 0.13379645401479928 … 1.0e-5 1.0e-5; 0.2227085563448121 0.45381280760422626 … 1.0e-5 1.0e-5], [0.582214916426889 1.0000000000027756e-5 … 0.9999699999999999 0.9999699999999999; 0.14092902448354105 0.41258807922799495 … 1.0e-5 1.0e-5; 0.05402188629156628 0.1339463980931601 … 1.0e-5 1.0e-5; 0.2228341727980035 0.45345552267884487 … 1.0e-5 1.0e-5], [0.794600708769161 0.8699629358466604 … 0.9327568172111603 0.99999; 0.6295417302123216 0.9882835240192088 … 0.8596468613040379 0.8877110943870553; 0.795332666017797 0.9004058223965158 … 0.9599270223437033 0.99999; 0.7314117939132884 0.8910746807079742 … 0.8107491631733151 0.99999], [0.7946379362185242 0.8699568858567608 … 0.932690502090477 0.99999; 0.6294705873253528 0.9882931595924863 … 0.8596295407288184 0.8876821133904721; 0.7947761216806404 0.9014162794343112 … 0.9604830519644512 0.99999; 0.7315585615635503 0.8908533540490032 … 0.8106984216220864 0.99999], [0.7946218752583306 0.8699668115606753 … 0.9326919414077695 0.99999; 0.629492931909354 0.9882919440582303 … 0.8596604973569697 0.8876695309831731; 0.7949439576510982 0.9012387456154918 … 0.9604303962397303 0.99999; 0.7315050094973324 0.890896060077284 … 0.8106865432804832 0.99999], [0.794600708769161 0.8699629358466604 … 0.9327568172111603 0.99999; 0.6295417302123216 0.9882835240192088 … 0.8596468613040379 0.8877110943870553; 0.795332666017797 0.9004058223965158 … 0.9599270223437033 0.99999; 0.7314117939132884 0.8910746807079742 … 0.8107491631733151 0.99999], [20324.008613992417 19517.205560497827 19445.052396609426 19593.620284729197; 19517.205560497827 21129.69564684323 19991.760457049983 20546.476546981514; 19445.052396609426 19991.760457049983 27061.91936795712 19733.76140174633; 19593.620284729197 20546.476546981514 19733.76140174633 20783.794235985824;;; 20537.320580307394 19532.61527210971 19904.474838050613 19816.064775376904; 19532.61527210971 20429.751878926836 19208.639123757057 19842.885390646392; 19904.474838050613 19208.639123757057 24043.105805147836 19529.028777521504; 19816.064775376904 19842.885390646392 19529.028777521504 20281.005129401456;;; 20554.33202737424 19527.167907738505 20146.012553921053 19827.729637940454; 19527.167907738512 20277.465892718952 19338.0657146504 19815.571658413777; 20146.01255392105 19338.0657146504 24509.245681961882 19667.149225394885; 19827.729637940458 19815.571658413777 19667.149225394885 20326.462531857924;;; … ;;; 20161.223377180504 19799.481341587463 19499.054021029686 19738.89607701232; 19799.481341587463 20880.864584502502 19303.322514294978 20163.93967566195; 19499.054021029686 19303.322514294978 22942.205947459446 19089.16440119291; 19738.89607701232 20163.93967566195 19089.16440119291 20285.72352733147;;; 20000.02114719698 19652.53691666853 19632.312142147246 19657.796437926452; 19652.53691666853 21076.84228656145 19791.029925493247 20375.34137743504; 19632.312142147246 19791.029925493247 24012.84358997869 19676.25199063923; 19657.796437926452 20375.34137743504 19676.25199063923 20499.803807281765;;; 20000.014106027465 19707.89979655304 19747.929550133194 19838.811482171674; 19707.89979655304 22045.021897104154 20875.403318563087 21565.27026454133; 19747.929550133194 20875.403318563087 25203.810149595418 20909.98526841688; 19838.811482171674 21565.270264541326 20909.98526841688 21840.790994511528], [-20000.0278902452 -19711.042676807927 … -20000.010573017542 -20000.007052012574; -19999.657638990368 -20000.018840334767 … -19652.559772696888 -19707.95341651394; -19999.54125456821 -20000.44382380299 … -19632.357974038034 -19748.00700423404; -20000.25732032506 -19999.858582666187 … -19657.81221800508 -19838.859478292485], [566.628571738167 83.90099875125914 62.70760160459943 206.69526708612761; 83.90099875125914 493.3179302661993 50.12892125483464 184.51024479111086; 62.70760160459943 50.12892125483464 103.82896536324253 202.09259891424378; 206.69526708612761 184.51024479111086 202.09259891424378 1153.4061405968855;;; 958.3641448625666 96.0657103671523 99.84997857047428 452.8669952750855; 96.0657103671523 8422.223541256804 173.75693630488144 541.6772331313043; 99.84997857047429 173.75693630488135 205.75467061768856 477.66509054457424; 452.8669952750855 541.6772331313044 477.66509054457424 2396.37473968966;;; 4787.553491320607 279.1023137653173 161.93765801627018 1347.7563046867367; 279.1023137653173 411.5348370695136 213.93028528940792 958.6175587313586; 161.93765801627018 213.93028528940792 411.75403584058546 1352.7814161727115; 1347.7563046867367 958.6175587313586 1352.7814161727115 7467.536092617086;;; … ;;; 650.1821214406797 81.84322018117557 59.023027883089135 273.76127310783073; 81.84322018117557 520.8311086730197 52.169654366007286 197.2850117549217; 59.02302788308914 52.169654366007286 333.9347030552088 270.49525897176716; 273.76127310783073 197.2850117549217 270.49525897176716 1339.7021125851502;;; 944.8744923000977 210.87018222789402 197.4712494119201 430.93057398367193; 210.87018222789402 935.5813002844052 105.64660374792102 292.4439054220507; 197.47124941192013 105.64660374792102 185.08439855823875 292.02256227961254; 430.93057398367193 292.44390542205076 292.02256227961254 1682.286841236985;;; 1047.3154837338877 785.9251092419285 309.8161299481653 241.61464140247605; 785.9251092419285 1708.5153389919187 485.3986936556763 765.6967877385282; 309.8161299481654 485.39869365567637 266.94660041782066 305.25080391581776; 241.61464140247605 765.6967877385282 305.25080391581776 886.2985403373846], [0.007770233480390409 -0.011009070285414424 … 0.007628928298945681 -76.78147650807796; -0.008207984630832166 0.016998830640420515 … -0.020229304424980027 0.0055804901744127555; -0.006716724522825013 0.015349359734320167 … 0.00965981047948361 -24.821817094463924; 0.027045794349309205 -0.021374536238511155 … 0.02568611436415935 -257.2160433157398], SubArray{Float64, 1, Matrix{Float64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[0.582214916426889, 0.14092902448354105, 0.05402188629156628, 0.2228341727980035], [1.0000000000027756e-5, 0.41258807922799495, 0.1339463980931601, 0.45345552267884487], [1.0e-5, 0.5069557466869863, 0.10389750859810855, 0.38913674471490506], [0.4475320713476874, 0.028059304873610785, 0.06263047002167084, 0.46177815375703096], [1.0000000000055512e-5, 0.3248282288404811, 0.11654224261353534, 0.5586195285459835], [1.0000000000027756e-5, 0.4937610338831034, 0.1284319653692813, 0.3777970007476152], [1.0e-5, 1.0e-5, 0.201860121489011, 0.798119878510989], [1.0e-5, 0.5683277274079043, 0.04197256837358268, 0.3896897042185129], [0.432818900266916, 0.12616848672266018, 1.0000000000027756e-5, 0.4410026130104237], [1.0e-5, 1.0e-5, 1.0e-5, 0.9999699999999999]  …  [0.9999699999999999, 1.0e-5, 1.0e-5, 1.0e-5], [0.20421694513583225, 1.0e-5, 0.7957630548641677, 1.0e-5], [1.0e-5, 0.21768156700974545, 0.22585943488504254, 0.556448998105212], [0.9999699999999999, 1.0e-5, 1.0e-5, 1.0e-5], [1.0e-5, 1.0e-5, 0.2670590285882958, 0.7329209714117042], [0.47701859813822794, 1.0e-5, 1.0e-5, 0.522961401861772], [0.3995928820841258, 1.0e-5, 0.14401354326622046, 0.4563835746496538], [1.0000000000055512e-5, 0.05193865569654182, 0.2338316743309882, 0.71421966997247], [0.9999699999999999, 1.0e-5, 1.0e-5, 1.0e-5], [0.9999699999999999, 1.0e-5, 1.0e-5, 1.0e-5]], SubArray{Float64, 1, Matrix{Float64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[0.5822686983061394, 0.14157978700364626, 0.05422353338551516, 0.2219279813046991], [1.0e-5, 0.4122387868461372, 0.13369030486485345, 0.45406090828900936], [1.0e-5, 0.5061059458488054, 0.10326456862507746, 0.3906194855261171], [0.44755321977495693, 0.027942585975311938, 0.06239837383851021, 0.462105820411221], [1.0e-5, 0.325005603335161, 0.11657610868601087, 0.5584082879788281], [1.0e-5, 0.4936202131430932, 0.1285433768248627, 0.37782641003204415], [1.0e-5, 1.0e-5, 0.20177013123801546, 0.7982098687619844], [1.0e-5, 0.567854974015666, 0.041866892126303186, 0.3902681338580307], [0.4328984655887788, 0.12661866186872353, 1.0e-5, 0.4404728725424976], [1.0e-5, 1.0e-5, 1.0e-5, 0.9999699999999999]  …  [0.9999699999999999, 1.0e-5, 1.0e-5, 1.0e-5], [0.20403525394233787, 1.0e-5, 0.795944746057662, 1.0e-5], [1.0e-5, 0.2175467131256098, 0.22553880954161065, 0.5569044773327795], [0.9999699999999999, 1.0e-5, 1.0e-5, 1.0e-5], [1.0e-5, 1.0e-5, 0.2663651985736278, 0.7336148014263721], [0.477083438661804, 1.0e-5, 1.0e-5, 0.522896561338196], [0.3996465219669129, 1.0e-5, 0.14411265962056485, 0.4562308184125222], [1.0e-5, 0.05168193530267726, 0.23354006083142623, 0.7147680038658963], [0.9999699999999999, 1.0e-5, 1.0e-5, 1.0e-5], [0.9999699999999999, 1.0e-5, 1.0e-5, 1.0e-5]], SubArray{Float64, 1, Matrix{Float64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[0.582214916426889, 0.14092902448354105, 0.05402188629156628, 0.2228341727980035], [1.0000000000027756e-5, 0.41258807922799495, 0.1339463980931601, 0.45345552267884487], [1.0e-5, 0.5069557466869863, 0.10389750859810855, 0.38913674471490506], [0.4475320713476874, 0.028059304873610785, 0.06263047002167084, 0.46177815375703096], [1.0000000000055512e-5, 0.3248282288404811, 0.11654224261353534, 0.5586195285459835], [1.0000000000027756e-5, 0.4937610338831034, 0.1284319653692813, 0.3777970007476152], [1.0e-5, 1.0e-5, 0.201860121489011, 0.798119878510989], [1.0e-5, 0.5683277274079043, 0.04197256837358268, 0.3896897042185129], [0.432818900266916, 0.12616848672266018, 1.0000000000027756e-5, 0.4410026130104237], [1.0e-5, 1.0e-5, 1.0e-5, 0.9999699999999999]  …  [0.9999699999999999, 1.0e-5, 1.0e-5, 1.0e-5], [0.20421694513583225, 1.0e-5, 0.7957630548641677, 1.0e-5], [1.0e-5, 0.21768156700974545, 0.22585943488504254, 0.556448998105212], [0.9999699999999999, 1.0e-5, 1.0e-5, 1.0e-5], [1.0e-5, 1.0e-5, 0.2670590285882958, 0.7329209714117042], [0.47701859813822794, 1.0e-5, 1.0e-5, 0.522961401861772], [0.3995928820841258, 1.0e-5, 0.14401354326622046, 0.4563835746496538], [1.0000000000055512e-5, 0.05193865569654182, 0.2338316743309882, 0.71421966997247], [0.9999699999999999, 1.0e-5, 1.0e-5, 1.0e-5], [0.9999699999999999, 1.0e-5, 1.0e-5, 1.0e-5]], SubArray{Float64, 1, Matrix{Float64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[0.794600708769161, 0.6295417302123216, 0.795332666017797, 0.7314117939132884], [0.8699629358466604, 0.9882835240192088, 0.9004058223965158, 0.8910746807079742], [0.9737349382463225, 0.99999, 0.8579504459370331, 0.9841868447992824], [0.9896365156352461, 0.99999, 0.9166339405314132, 0.9979607677791603], [0.6468518890092232, 0.6478882446742358, 0.494559790425589, 0.5599804804012329], [0.6743182225242905, 0.6472794791502667, 0.7235288407693508, 0.6734354662135091], [0.7469737872889323, 0.7039895414260632, 0.8789128988015523, 0.6672052923426718], [0.8607860295663952, 0.855127530928302, 0.5702185709368484, 0.9129512026702861], [0.8519524721903828, 0.8376414128223882, 0.5892841242129675, 0.8840536068250283], [0.5328553708993604, 0.46118733987300753, 0.3879047896303323, 0.553649639999572]  …  [0.9620399261441169, 0.9861773938986236, 0.9256286539109566, 0.9839677062672065], [0.8741312762540471, 0.6799753331701319, 0.6678787641267006, 0.8799493479739465], [0.9302414910591702, 0.9094934930781865, 0.8561830373420675, 0.9697104426569796], [0.5921661766180285, 0.7758305587523612, 0.822145442576713, 0.6655953985078696], [0.9682851330624743, 0.973439956988747, 0.9503928900333226, 0.9432971849309506], [0.8418753575433897, 0.9795046963142533, 0.73576110676302, 0.8590486768118991], [0.934462853272321, 0.7354040200346937, 0.99999, 0.8470050399575564], [0.8322509557951396, 0.639827870913738, 0.9785293750103126, 0.7676496020277787], [0.9327568172111603, 0.8596468613040379, 0.9599270223437033, 0.8107491631733151], [0.99999, 0.8877110943870553, 0.99999, 0.99999]], SubArray{Float64, 1, Matrix{Float64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[0.7946379362185242, 0.6294705873253528, 0.7947761216806404, 0.7315585615635503], [0.8699568858567608, 0.9882931595924863, 0.9014162794343112, 0.8908533540490032], [0.9737288831070886, 0.99999, 0.8583960664737883, 0.9840594249932237], [0.9896451023631544, 0.99999, 0.916643911983247, 0.9979374543131986], [0.6467656418485923, 0.6479694526127282, 0.49541475699629417, 0.559774837963584], [0.6743605643204243, 0.6472086404740693, 0.7228263739828681, 0.6736236500461458], [0.7469561522385367, 0.7040609955778488, 0.8800527339886484, 0.6669619900938552], [0.8607443290730644, 0.8551055746339635, 0.570778125418224, 0.9127684360300229], [0.8519045526131043, 0.8376074684093889, 0.5896231667925191, 0.8839468953184086], [0.5328993281224814, 0.4610079681118647, 0.38537291133768914, 0.5542425554896754]  …  [0.9620904238107759, 0.9861790045306884, 0.9253944281744522, 0.9839850914887557], [0.8741633166232774, 0.6799008864771244, 0.6682967903170592, 0.8798032701585558], [0.9302834980465161, 0.9094488829788975, 0.8556166814167883, 0.9698126899072685], [0.5921917731312764, 0.7759738840219016, 0.822008243916343, 0.6655876537132536], [0.9682908656598623, 0.9734716336131075, 0.9502424124591029, 0.9433182434551405], [0.841859229091953, 0.9796269382104448, 0.7356924340905762, 0.8590041696522682], [0.9344774585661986, 0.7354502518248635, 0.99999, 0.846984895742093], [0.8322734576267384, 0.6397921048838265, 0.9782466651408858, 0.7677475795250771], [0.932690502090477, 0.8596295407288184, 0.9604830519644512, 0.8106984216220864], [0.99999, 0.8876821133904721, 0.99999, 0.99999]], SubArray{Float64, 1, Matrix{Float64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[0.794600708769161, 0.6295417302123216, 0.795332666017797, 0.7314117939132884], [0.8699629358466604, 0.9882835240192088, 0.9004058223965158, 0.8910746807079742], [0.9737349382463225, 0.99999, 0.8579504459370331, 0.9841868447992824], [0.9896365156352461, 0.99999, 0.9166339405314132, 0.9979607677791603], [0.6468518890092232, 0.6478882446742358, 0.494559790425589, 0.5599804804012329], [0.6743182225242905, 0.6472794791502667, 0.7235288407693508, 0.6734354662135091], [0.7469737872889323, 0.7039895414260632, 0.8789128988015523, 0.6672052923426718], [0.8607860295663952, 0.855127530928302, 0.5702185709368484, 0.9129512026702861], [0.8519524721903828, 0.8376414128223882, 0.5892841242129675, 0.8840536068250283], [0.5328553708993604, 0.46118733987300753, 0.3879047896303323, 0.553649639999572]  …  [0.9620399261441169, 0.9861773938986236, 0.9256286539109566, 0.9839677062672065], [0.8741312762540471, 0.6799753331701319, 0.6678787641267006, 0.8799493479739465], [0.9302414910591702, 0.9094934930781865, 0.8561830373420675, 0.9697104426569796], [0.5921661766180285, 0.7758305587523612, 0.822145442576713, 0.6655953985078696], [0.9682851330624743, 0.973439956988747, 0.9503928900333226, 0.9432971849309506], [0.8418753575433897, 0.9795046963142533, 0.73576110676302, 0.8590486768118991], [0.934462853272321, 0.7354040200346937, 0.99999, 0.8470050399575564], [0.8322509557951396, 0.639827870913738, 0.9785293750103126, 0.7676496020277787], [0.9327568172111603, 0.8596468613040379, 0.9599270223437033, 0.8107491631733151], [0.99999, 0.8877110943870553, 0.99999, 0.99999]], SubArray{Float64, 2, Array{Float64, 3}, Tuple{Base.Slice{Base.OneTo{Int64}}, Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[20324.008613992417 19517.205560497827 19445.052396609426 19593.620284729197; 19517.205560497827 21129.69564684323 19991.760457049983 20546.476546981514; 19445.052396609426 19991.760457049983 27061.91936795712 19733.76140174633; 19593.620284729197 20546.476546981514 19733.76140174633 20783.794235985824], [20537.320580307394 19532.61527210971 19904.474838050613 19816.064775376904; 19532.61527210971 20429.751878926836 19208.639123757057 19842.885390646392; 19904.474838050613 19208.639123757057 24043.105805147836 19529.028777521504; 19816.064775376904 19842.885390646392 19529.028777521504 20281.005129401456], [20554.33202737424 19527.167907738505 20146.012553921053 19827.729637940454; 19527.167907738512 20277.465892718952 19338.0657146504 19815.571658413777; 20146.01255392105 19338.0657146504 24509.245681961882 19667.149225394885; 19827.729637940458 19815.571658413777 19667.149225394885 20326.462531857924], [20556.41070737937 19564.363129783676 19569.14953476913 19545.65979654395; 19564.363129783676 21160.217133344173 20156.301774785272 20330.734108958746; 19569.14953476913 20156.301774785272 25935.389042991374 19607.66575933755; 19545.65979654395 20330.734108958746 19607.66575933755 20472.803926885408], [20738.68452415396 19547.04190311882 19986.87333496084 19774.520712450467; 19547.04190311882 20607.449826191758 19367.81446189019 19778.27309676111; 19986.87333496084 19367.81446189019 24115.635312216564 19508.22326067277; 19774.520712450467 19778.27309676111 19508.22326067277 20231.933071780884], [29115.766672205205 19736.911593229335 20025.31836148489 19962.334757940967; 19736.911593229335 20298.33626249491 19343.40982603822 19833.675560129097; 20025.31836148489 19343.40982603822 23889.466523741874 19533.288493945907; 19962.33475794097 19833.675560129097 19533.288493945907 20376.452799022445], [20324.696601491036 19768.31715243907 19571.894649782054 19731.684938919876; 19768.317152439075 20806.688816268073 19453.77420120726 20082.307620287003; 19571.894649782054 19453.774201207263 23346.79055965274 19154.314509536325; 19731.684938919876 20082.307620287003 19154.314509536325 20213.70327499686], [21086.11788854685 19524.422635720646 21136.020827604458 19972.3878700753; 19524.422635720646 20188.699167270217 19567.87966176365 19771.92454600974; 21136.020827604458 19567.87966176365 26788.80122868821 19901.80305390232; 19972.3878700753 19771.924546009737 19901.80305390232 20342.087580709125], [20630.330394418503 19498.715545835796 19681.119243186386 19524.670512182012; 19498.715545835796 20923.8424877803 19733.139316755387 20226.49185506534; 19681.119243186386 19733.139316755387 23969.02414737194 19718.648704449308; 19524.670512182012 20226.49185506534 19718.648704449308 20402.191260865442], [20446.465987621606 19845.666905542515 20419.297927072817 19648.570545431245; 19845.666905542515 20846.783846009883 20066.018857225496 19968.892383880248; 20419.297927072817 20066.018857225496 25251.211991353313 19914.93021724633; 19648.570545431245 19968.892383880248 19914.93021724633 20000.00935169697]  …  [20000.012013136115 19694.044998495727 19993.029393624973 19712.228872456348; 19694.044998495727 21304.70736265766 20559.027111921965 20518.570359894573; 19993.02939362497 20559.027111921965 27496.19528042929 19992.066301000348; 19712.228872456348 20518.57035989457 19992.066301000348 20649.219381936982], [24467.79577912119 20911.8650341461 18855.309605645212 20991.870132511227; 20911.8650341461 22304.30536982939 19119.58529600222 21788.03937092589; 18855.309605645212 19119.58529600222 20293.313019723617 19083.54212225295; 20991.870132511227 21788.03937092589 19083.54212225295 22029.958866969235], [20265.720529504815 19626.26120331209 19467.630344483172 19816.551302234795; 19626.261203312093 20947.046472290487 19170.429288011834 19966.055578938172; 19467.630344483172 19170.429288011834 22583.40605282289 19278.364551434977; 19816.551302234795 19966.055578938172 19278.364551434977 20305.293737867087], [20000.016250475015 19583.30882916866 19929.17880271616 19674.979977249706; 19583.30882916866 20833.69645194361 19910.599207547042 20318.859656570818; 19929.17880271616 19910.599207547042 24830.29519993198 19966.31480264646; 19674.979977249706 20318.859656570818 19966.31480264646 20547.415910226428], [22603.95026169848 20053.11149815297 19581.497305905228 19880.160560723336; 20053.11149815297 20976.490812875414 19214.835893986507 20232.15791907455; 19581.497305905228 19214.835893986507 22625.62466062325 19048.012007653953; 19880.160560723336 20232.15791907455 19048.012007653953 20345.17598759747], [20530.08071498536 19605.211692440156 20045.941897415705 19516.404624214025; 19605.211692440156 20998.886009443206 20209.15875088167 20307.40599497684; 20045.941897415705 20209.15875088167 25722.11215343645 19941.747516334442; 19516.404624214025 20307.405994976838 19941.74751633444 20441.189198610202], [20666.64864225557 19526.766568006435 19345.291052039345 19622.92899416531; 19526.766568006435 21482.104669623564 19783.013056694053 20223.771547515877; 19345.291052039345 19783.013056694053 23415.675962667327 19493.86923556915; 19622.92899416531 20223.771547515877 19493.86923556915 20490.336207422664], [20161.223377180504 19799.481341587463 19499.054021029686 19738.89607701232; 19799.481341587463 20880.864584502502 19303.322514294978 20163.93967566195; 19499.054021029686 19303.322514294978 22942.205947459446 19089.16440119291; 19738.89607701232 20163.93967566195 19089.16440119291 20285.72352733147], [20000.02114719698 19652.53691666853 19632.312142147246 19657.796437926452; 19652.53691666853 21076.84228656145 19791.029925493247 20375.34137743504; 19632.312142147246 19791.029925493247 24012.84358997869 19676.25199063923; 19657.796437926452 20375.34137743504 19676.25199063923 20499.803807281765], [20000.014106027465 19707.89979655304 19747.929550133194 19838.811482171674; 19707.89979655304 22045.021897104154 20875.403318563087 21565.27026454133; 19747.929550133194 20875.403318563087 25203.810149595418 20909.98526841688; 19838.811482171674 21565.270264541326 20909.98526841688 21840.790994511528]], SubArray{Float64, 1, Matrix{Float64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[-20000.0278902452, -19999.657638990368, -19999.54125456821, -20000.25732032506], [-19711.042676807927, -20000.018840334767, -20000.44382380299, -19999.858582666187], [-19708.488173234004, -20000.02674741278, -20000.619894388008, -19999.80893161833], [-20000.01296204302, -20000.035916413508, -20000.59765104925, -19999.904573253116], [-19725.353708817314, -19999.908197239423, -19999.708568893617, -20000.119190170106], [-19859.248986731913, -20000.02009442893, -19999.523299720917, -20000.13965449756], [-19699.45032774874, -19955.492453739073, -20000.23812277887, -19999.944130617892], [-19766.737416368596, -20000.046887559227, -20000.532546571743, -19999.88062366939], [-20000.024160044974, -19999.731277950286, -19704.279543948513, -20000.06021615895], [-19648.588202623083, -19968.900901804824, -19914.990134627566, -20000.004675347882]  …  [-20000.006005808373, -19694.078000194124, -19993.110075630073, -19712.249104150604], [-20000.496578457794, -19485.33207925158, -19999.885792975976, -19472.96482157121], [-19696.46359093068, -20000.018918953847, -20000.300608013516, -19999.876317809518], [-20000.00812466358, -19583.331961456952, -19929.22799944419, -19674.9980537541], [-19800.63603095345, -19961.184388713365, -20000.970515473484, -19999.650866162294], [-20000.018882731878, -19972.40665484161, -19991.517407708634, -19999.983461587013], [-20000.035817557484, -19881.709636629846, -19999.67537067944, -20000.07376033089], [-19686.018735665253, -19999.99951361597, -20000.076162936093, -19999.979542807254], [-20000.010573017542, -19652.559772696888, -19632.357974038034, -19657.81221800508], [-20000.007052012574, -19707.95341651394, -19748.00700423404, -19838.859478292485]], SubArray{Float64, 2, Array{Float64, 3}, Tuple{Base.Slice{Base.OneTo{Int64}}, Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[566.628571738167 83.90099875125914 62.70760160459943 206.69526708612761; 83.90099875125914 493.3179302661993 50.12892125483464 184.51024479111086; 62.70760160459943 50.12892125483464 103.82896536324253 202.09259891424378; 206.69526708612761 184.51024479111086 202.09259891424378 1153.4061405968855], [958.3641448625666 96.0657103671523 99.84997857047428 452.8669952750855; 96.0657103671523 8422.223541256804 173.75693630488144 541.6772331313043; 99.84997857047429 173.75693630488135 205.75467061768856 477.66509054457424; 452.8669952750855 541.6772331313044 477.66509054457424 2396.37473968966], [4787.553491320607 279.1023137653173 161.93765801627018 1347.7563046867367; 279.1023137653173 411.5348370695136 213.93028528940792 958.6175587313586; 161.93765801627018 213.93028528940792 411.75403584058546 1352.7814161727115; 1347.7563046867367 958.6175587313586 1352.7814161727115 7467.536092617086], [7767.178149087687 3862.697527547379 920.8781344728436 4829.563439493999; 3862.6975275473797 9554.786331558318 1177.9110050688953 6700.0341545802585; 920.8781344728436 1177.9110050688953 783.4548895988855 3020.6659537625974; 4829.563439493999 6700.0341545802585 3020.6659537625974 19283.020349423085], [410.42566951077333 69.19386885522066 47.63981368130034 166.26081696105726; 69.19386885522066 470.73724653646144 46.57125357358593 162.3642840607006; 47.63981368130034 46.57125357358593 86.99245751330125 148.4481762818415; 166.26081696105726 162.3642840607006 148.4481762818415 906.7796047602883], [441.6608378423226 67.8182524868344 45.58456363198682 196.36088728498288; 67.81825248683438 488.19032217377 47.412403385162335 179.80911344251018; 45.58456363198682 47.412403385162335 116.21441409086555 162.32181701959905; 196.36088728498288 179.80911344251015 162.32181701959905 1002.402513083182], [503.9229298358099 80.39658980374581 58.874083051406146 199.85393717343624; 80.39658980374581 539.4246098890644 55.11026961290292 184.30301365371213; 58.874083051406146 55.11026961290292 108.41042355622672 187.17230190911135; 199.85393717343624 184.30301365371213 187.17230190911135 1065.047904070726], [849.430931507485 93.74875249803007 81.091245853374 294.374847145811; 93.74875249803007 884.2497375684054 82.15283012480859 344.76286746770626; 81.091245853374 82.15283012480859 124.57520279265023 274.5750308482612; 294.374847145811 344.76286746770626 274.57503084826124 2106.5862234020346], [803.0756671298689 94.30617516935874 77.65861720413116 250.87872248055902; 94.30617516935874 789.8248193423992 81.23373077083103 298.8790967047208; 77.65861720413116 81.23373077083103 119.16107309648585 255.85138132750058; 250.87872248055905 298.8790967047208 255.85138132750058 1813.8260832494495], [398.8832417694709 60.480381626956614 43.20059800292722 159.6623594136319; 60.480381626956614 443.60375678229354 44.797296067346906 159.3399382436201; 43.20059800292722 44.797296067346906 75.10094630562021 145.68448614315943; 159.6623594136319 159.3399382436201 145.68448614315943 899.0940015170539]  …  [3018.072020431244 202.71172670529344 359.66963498405073 1663.6436721776724; 202.71172670529344 6186.653880166012 674.968457907945 2243.8190247219572; 359.66963498405073 674.968457907945 514.3018209982837 1283.2142114964613; 1663.6436721776724 2243.8190247219572 1283.2142114964613 9783.241072487388], [787.5047787921353 94.28435314400008 67.33624711829918 374.1155690677025; 94.28435314400008 566.9581944937457 56.15164653798182 222.24141714800865; 67.33624711829918 56.151646537981826 124.34313686619643 273.9116929337594; 374.1155690677026 222.24141714800865 273.9116929337594 1633.1478321182278], [1627.669953546169 255.78823966607212 141.1003459592787 660.9850428023368; 255.7882396660721 1489.9176285904578 154.1583165859123 506.9057450087949; 141.1003459592787 154.1583165859123 280.6649939459262 691.7769382351324; 660.9850428023369 506.9057450087949 691.7769382351324 5511.4760199851025], [431.7817592172977 67.98943118540842 50.96925760895643 174.1181117861671; 67.98943118540842 583.5463251115096 53.69039099167622 209.8953893774308; 50.96925760895643 53.69039099167622 101.28300981373619 177.13140541099423; 174.1181117861671 209.8953893774308 177.13140541099423 1033.2935461177208], [2912.28067234033 290.6105762344447 137.25232229757876 1141.2183757896164; 290.6105762344447 3443.7686511850484 217.09733969883405 1248.0203305948573; 137.25232229757876 217.09733969883402 832.9418782781245 554.8820210009371; 1141.2183757896164 1248.0203305948573 554.8820210009371 4569.174432194636], [821.2137333477225 144.16699671408674 76.18016516850544 293.1195042935607; 144.16699671408674 3816.5909425829636 111.1201934991942 440.5113842872693; 76.18016516850544 111.1201934991942 145.79868310117928 287.76342276551594; 293.1195042935607 440.5113842872693 287.76342276551594 1856.0394038107715], [1306.5791208861426 112.15748161968828 112.6510737025306 445.515693993082; 112.15748161968827 649.906612484158 72.63588753838884 273.4773785992364; 112.65107370253061 72.63588753838884 118.85161378623019 353.3327017080312; 445.515693993082 273.4773785992364 353.3327017080312 1914.788970412585], [650.1821214406797 81.84322018117557 59.023027883089135 273.76127310783073; 81.84322018117557 520.8311086730197 52.169654366007286 197.2850117549217; 59.02302788308914 52.169654366007286 333.9347030552088 270.49525897176716; 273.76127310783073 197.2850117549217 270.49525897176716 1339.7021125851502], [944.8744923000977 210.87018222789402 197.4712494119201 430.93057398367193; 210.87018222789402 935.5813002844052 105.64660374792102 292.4439054220507; 197.47124941192013 105.64660374792102 185.08439855823875 292.02256227961254; 430.93057398367193 292.44390542205076 292.02256227961254 1682.286841236985], [1047.3154837338877 785.9251092419285 309.8161299481653 241.61464140247605; 785.9251092419285 1708.5153389919187 485.3986936556763 765.6967877385282; 309.8161299481654 485.39869365567637 266.94660041782066 305.25080391581776; 241.61464140247605 765.6967877385282 305.25080391581776 886.2985403373846]], SubArray{Float64, 1, Matrix{Float64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[0.007770233480390409, -0.008207984630832166, -0.006716724522825013, 0.027045794349309205], [-0.011009070285414424, 0.016998830640420515, 0.015349359734320167, -0.021374536238511155], [-0.06747874196181058, -126.47533832619307, 0.002986716228887687, -0.12732992969222678], [-0.11511737242070552, -24.927226237445016, -0.021825917181970134, -0.12852949450422813], [-0.009264326037800918, 0.011992961001884073, 0.011363580100436366, -0.025655309241940927], [0.007299056543518034, -0.019566503671352375, -0.013120364383844674, 0.032159741088385374], [0.008690135363860207, 0.015595850445510706, 0.014869786322051048, -0.013667826322333099], [-0.019000450771105903, 0.012470765595396172, 0.01019042457955699, -0.10512913155584869], [-0.016007075414943728, 0.014075384475613184, 0.005652536637894556, -0.061913702990143094], [-0.001941844666859538, -0.027969343086239773, -0.01892587331633866, 0.042920050965896905]  …  [0.013627363339328014, -0.020993829347255133, -0.01117067362963109, -0.04375485546886715], [0.00911496751036367, -0.004003612009534585, -0.008856384349900992, 0.01764736165946701], [0.009770035240974195, -0.009433777044988378, -0.017536518527551337, 0.04064013568779501], [0.0014454487025630236, 0.021321909070572964, 0.002708061115555438, -0.01781383827045424], [0.01428882385254937, 0.028443838404892396, -0.012617229914278827, -0.01815425325777653], [-0.017106663099349717, 0.0962303645699476, 0.0009694209913120311, -0.03637112041325086], [0.013776619148526237, 0.005453621113234108, -9.138562932677196, 0.019422970935315753], [0.014351748040734691, -0.004095098949083109, -0.05212730104687857, 0.041335159944987154], [0.007628928298945681, -0.020229304424980027, 0.00965981047948361, 0.02568611436415935], [-76.78147650807796, 0.0055804901744127555, -24.821817094463924, -257.2160433157398]], [0.9749512513803503 0.9749512513803503 … 0.0 0.0; 0.981569903892765 0.981569903892765 … 0.0 0.0; … ; 0.0 0.0 … 0.0 0.0; 0.0 0.0 … 0.0 0.0;;;], SubArray{Float64, 2, Array{Float64, 3}, Tuple{Base.Slice{Base.OneTo{Int64}}, Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[0.9749512513803503 0.9749512513803503 … 0.0 0.0; 0.981569903892765 0.981569903892765 … 0.0 0.0; … ; 0.0 0.0 … 0.0 0.0; 0.0 0.0 … 0.0 0.0]], [-0.00019912353358042534 -8.786143277184078e-5 0.0003996715928610284; -0.001010710455337599 -0.0008883743420693424 0.002671966296975248; … ; 0.0 0.0 0.0; 0.0 0.0 0.0], [-0.00011283827391972423 -4.233761613736675e-5 0.00019677106400273203; -0.0007188156708032578 -0.0006986276927519464 0.001696388905521179; … ; 0.0 0.0 0.0; 0.0 0.0 0.0], [-0.5 -0.5 -0.5 -0.5; -0.5 0.8333333333333334 -0.16666666666666666 -0.16666666666666666; -0.5 -0.16666666666666666 0.8333333333333334 -0.16666666666666666; -0.5 -0.16666666666666666 -0.16666666666666666 0.8333333333333334], [0.0; 0.0; 0.0; 0.0;;], [1047.3154837338877; 1708.5153389919187; … ; 886.2985403373846; 0.0;;], [0.6705595953304453; 0.0022016494740073954; … ; 0.6797167005044513; 0.1923469393993006;;], [20000.014106027465; 22045.021897104154; … ; 0.0; 0.0;;], [0.16076233081185012; 0.046115945456698504; … ; -0.17815723178570345; -0.017383501184341836;;], [685.7862289240434 0.4600047136279448 … -110.60949016702281 76.78131250930491; 0.4600047136279447 -0.0005853034954839992 … 0.44816500634423034 -0.0; … ; -110.60949016702281 0.44816500634423034 … 543.1400346027904 257.21271509888857; 76.78131250930491 -0.0 … 257.21271509888857 -0.006306245873800798;;;], [2163.1821361956468 144.60904435156553 … -2001.9795123398549 161.09418147682655; 144.60904435156553 755.2723625330268 … -275.52072997020105 -130.90032710809507; … ; -2001.9795123398549 -275.52072997020105 … 21840.790994511528 19838.90594006875; 161.09418147682655 -130.90032710809507 … 19838.90594006875 -0.0138550945177498;;;], Bool[0; 1; 0; 0;;], SubArray{Float64, 1, Matrix{Float64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[0.0, 0.0, 0.0, 0.0]], SubArray{Float64, 1, Matrix{Float64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[1047.3154837338877, 1708.5153389919187, 266.94660041782066, 886.2985403373846, 0.0]], SubArray{Float64, 1, Matrix{Float64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[0.6705595953304453, 0.0022016494740073954, 1.0, 0.6797167005044513, 0.1923469393993006]], SubArray{Float64, 1, Matrix{Float64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[20000.014106027465, 22045.021897104154, 25203.810149595418, 21840.790994511528, 0.0, 0.0]], SubArray{Float64, 1, Matrix{Float64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[0.16076233081185012, 0.046115945456698504, 1.0, 0.00019140109132460083, -0.17815723178570345, -0.017383501184341836]], SubArray{Float64, 2, Array{Float64, 3}, Tuple{Base.Slice{Base.OneTo{Int64}}, Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[685.7862289240434 0.4600047136279448 … -110.60949016702281 76.78131250930491; 0.4600047136279447 -0.0005853034954839992 … 0.44816500634423034 -0.0; … ; -110.60949016702281 0.44816500634423034 … 543.1400346027904 257.21271509888857; 76.78131250930491 -0.0 … 257.21271509888857 -0.006306245873800798]], SubArray{Float64, 2, Array{Float64, 3}, Tuple{Base.Slice{Base.OneTo{Int64}}, Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[2163.1821361956468 144.60904435156553 … -2001.9795123398549 161.09418147682655; 144.60904435156553 755.2723625330268 … -275.52072997020105 -130.90032710809507; … ; -2001.9795123398549 -275.52072997020105 … 21840.790994511528 19838.90594006875; 161.09418147682655 -130.90032710809507 … 19838.90594006875 -0.0138550945177498]], SubArray{Bool, 1, Matrix{Bool}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[0, 1, 0, 0]], [1; 2; 3; 4;;], SubArray{Int64, 1, Matrix{Int64}, Tuple{Base.Slice{Base.OneTo{Int64}}, Int64}, true}[[1, 2, 3, 4]], -2.9888459670782355e6, -2.9888456882726927e6), [1, 2, 2, 2, 2, 2, 2, 2, 2, 2  …  1, 4, 2, 1, 2, 2, 2, 2, 1, 1], [5800, 5691, 5677, 5688, 5775, 5769, 5668, 5673, 5674, 5675  …  29537, 14293, 30551, 23892, 35746, 35747, 3783, 40709, 50996, 4707])



The following shows the clustering result of the samples:


```julia
clusters
```




    379-element Vector{Int64}:
     1
     2
     2
     2
     2
     2
     2
     2
     2
     2
     1
     2
     2
     ⋮
     2
     2
     1
     4
     2
     1
     2
     2
     2
     2
     1
     1



And the following shows the SNPs selected.


```julia
aims
```




    10000-element Vector{Int64}:
      5800
      5691
      5677
      5688
      5775
      5769
      5668
      5673
      5674
      5675
      5676
      5679
      5683
         ⋮
     12387
     21962
     29537
     14293
     30551
     23892
     35746
     35747
      3783
     40709
     50996
      4707



The allele frequencies can be shown as in: 


```julia
d.p
```




    4×10000 Matrix{Float64}:
     0.794601  0.869963  0.973735  0.989637  …  0.832251  0.932757  0.99999
     0.629542  0.988284  0.99999   0.99999      0.639828  0.859647  0.887711
     0.795333  0.900406  0.85795   0.916634     0.978529  0.959927  0.99999
     0.731412  0.891075  0.984187  0.997961     0.76765   0.810749  0.99999



!!! The order of alleles is in the order of index (as in `sort(aim)`). This can be verified by checking the `.bim` file generated along with the newly filtered `.bed` file. 

The admixture proportions can be viewed by:


```julia
d.q
```




    4×379 Matrix{Float64}:
     0.582215   1.0e-5    1.0e-5    0.447532   …  1.0e-5     0.99997  0.99997
     0.140929   0.412588  0.506956  0.0280593     0.0519387  1.0e-5   1.0e-5
     0.0540219  0.133946  0.103898  0.0626305     0.233832   1.0e-5   1.0e-5
     0.222834   0.453456  0.389137  0.461778      0.71422    1.0e-5   1.0e-5




```julia
d.ll_new
```




    -2.9888456882726927e6



## Multi-threading

Multi-threading is supported by default if the julia session is launched with multiple threads. For example, one may launch Julia with 8 thread using the following command: 
```bash
julia -t 8
```

## GPU support
GPU is enabled by setting the keyword argument `use_gpu` to `true`. The parts computing gradients and Hessians of the loglikelihood is moved to GPU.
